// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: yandex/cloud/iam/v1/access_analyzer_service.proto

package iam

import (
	_ "github.com/yandex-cloud/go-genproto/yandex/cloud"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ListSubjectAccessBindingsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// SubjectId to list access bindings for.
	SubjectId string `protobuf:"bytes,1,opt,name=subject_id,json=subjectId,proto3" json:"subject_id,omitempty"`
	// OrganizationId to search access bindings in.
	OrganizationId string `protobuf:"bytes,2,opt,name=organization_id,json=organizationId,proto3" json:"organization_id,omitempty"`
	// The maximum number of results per page to return. If the number of available
	// results is larger than [page_size],
	// the service returns a [ListSubjectAccessBindingsResponse.next_page_token]
	// that can be used to get the next page of results in subsequent list requests.
	// Default value: 100
	PageSize int64 `protobuf:"varint,3,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Page token. To get the next page of results, set [page_token]
	// to the [ListSubjectAccessBindingsResponse.next_page_token]
	// returned by a previous list request.
	PageToken     string `protobuf:"bytes,4,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListSubjectAccessBindingsRequest) Reset() {
	*x = ListSubjectAccessBindingsRequest{}
	mi := &file_yandex_cloud_iam_v1_access_analyzer_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListSubjectAccessBindingsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListSubjectAccessBindingsRequest) ProtoMessage() {}

func (x *ListSubjectAccessBindingsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_access_analyzer_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListSubjectAccessBindingsRequest.ProtoReflect.Descriptor instead.
func (*ListSubjectAccessBindingsRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDescGZIP(), []int{0}
}

func (x *ListSubjectAccessBindingsRequest) GetSubjectId() string {
	if x != nil {
		return x.SubjectId
	}
	return ""
}

func (x *ListSubjectAccessBindingsRequest) GetOrganizationId() string {
	if x != nil {
		return x.OrganizationId
	}
	return ""
}

func (x *ListSubjectAccessBindingsRequest) GetPageSize() int64 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListSubjectAccessBindingsRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

type ListSubjectAccessBindingsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// List of access bindings for the specified subject.
	SubjectAccessBindings []*SubjectAccessBinding `protobuf:"bytes,1,rep,name=subject_access_bindings,json=subjectAccessBindings,proto3" json:"subject_access_bindings,omitempty"`
	// This token allows you to get the next page of results for list requests. If the number of results
	// is larger than [ListSubjectAccessBindingsRequest.page_size], use
	// the [next_page_token] as the value
	// for the [ListSubjectAccessBindingsRequest.page_token] query parameter
	// in the next list request. Each subsequent list request will have its own
	// [next_page_token] to continue paging through the results.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListSubjectAccessBindingsResponse) Reset() {
	*x = ListSubjectAccessBindingsResponse{}
	mi := &file_yandex_cloud_iam_v1_access_analyzer_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListSubjectAccessBindingsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListSubjectAccessBindingsResponse) ProtoMessage() {}

func (x *ListSubjectAccessBindingsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_access_analyzer_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListSubjectAccessBindingsResponse.ProtoReflect.Descriptor instead.
func (*ListSubjectAccessBindingsResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDescGZIP(), []int{1}
}

func (x *ListSubjectAccessBindingsResponse) GetSubjectAccessBindings() []*SubjectAccessBinding {
	if x != nil {
		return x.SubjectAccessBindings
	}
	return nil
}

func (x *ListSubjectAccessBindingsResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

type SubjectAccessBinding struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Role assigned to the subject.
	RoleId string `protobuf:"bytes,1,opt,name=role_id,json=roleId,proto3" json:"role_id,omitempty"`
	// Resource to which the access binding applies.
	Resource *Resource `protobuf:"bytes,2,opt,name=resource,proto3" json:"resource,omitempty"`
	// Identifier of the subject to which this role is granted.
	SubjectId     string `protobuf:"bytes,3,opt,name=subject_id,json=subjectId,proto3" json:"subject_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SubjectAccessBinding) Reset() {
	*x = SubjectAccessBinding{}
	mi := &file_yandex_cloud_iam_v1_access_analyzer_service_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SubjectAccessBinding) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SubjectAccessBinding) ProtoMessage() {}

func (x *SubjectAccessBinding) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_access_analyzer_service_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SubjectAccessBinding.ProtoReflect.Descriptor instead.
func (*SubjectAccessBinding) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDescGZIP(), []int{2}
}

func (x *SubjectAccessBinding) GetRoleId() string {
	if x != nil {
		return x.RoleId
	}
	return ""
}

func (x *SubjectAccessBinding) GetResource() *Resource {
	if x != nil {
		return x.Resource
	}
	return nil
}

func (x *SubjectAccessBinding) GetSubjectId() string {
	if x != nil {
		return x.SubjectId
	}
	return ""
}

var File_yandex_cloud_iam_v1_access_analyzer_service_proto protoreflect.FileDescriptor

const file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDesc = "" +
	"\n" +
	"1yandex/cloud/iam/v1/access_analyzer_service.proto\x12\x13yandex.cloud.iam.v1\x1a\x1cgoogle/api/annotations.proto\x1a\"yandex/cloud/iam/v1/resource.proto\x1a\x1dyandex/cloud/validation.proto\"\xdb\x01\n" +
	" ListSubjectAccessBindingsRequest\x12,\n" +
	"\n" +
	"subject_id\x18\x01 \x01(\tB\r\xe8\xc71\x01\x8a\xc81\x05<=100R\tsubjectId\x125\n" +
	"\x0forganization_id\x18\x02 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x0eorganizationId\x12'\n" +
	"\tpage_size\x18\x03 \x01(\x03B\n" +
	"\xfa\xc71\x06<=1000R\bpageSize\x12)\n" +
	"\n" +
	"page_token\x18\x04 \x01(\tB\n" +
	"\x8a\xc81\x06<=2000R\tpageToken\"\xae\x01\n" +
	"!ListSubjectAccessBindingsResponse\x12a\n" +
	"\x17subject_access_bindings\x18\x01 \x03(\v2).yandex.cloud.iam.v1.SubjectAccessBindingR\x15subjectAccessBindings\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\x89\x01\n" +
	"\x14SubjectAccessBinding\x12\x17\n" +
	"\arole_id\x18\x01 \x01(\tR\x06roleId\x129\n" +
	"\bresource\x18\x02 \x01(\v2\x1d.yandex.cloud.iam.v1.ResourceR\bresource\x12\x1d\n" +
	"\n" +
	"subject_id\x18\x03 \x01(\tR\tsubjectId2\xcf\x01\n" +
	"\x15AccessAnalyzerService\x12\xb5\x01\n" +
	"\x19ListSubjectAccessBindings\x125.yandex.cloud.iam.v1.ListSubjectAccessBindingsRequest\x1a6.yandex.cloud.iam.v1.ListSubjectAccessBindingsResponse\")\x82\xd3\xe4\x93\x02#\x12!/iam/v1/listSubjectAccessBindingsBV\n" +
	"\x17yandex.cloud.api.iam.v1Z;github.com/yandex-cloud/go-genproto/yandex/cloud/iam/v1;iamb\x06proto3"

var (
	file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDescOnce sync.Once
	file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDescData []byte
)

func file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDescGZIP() []byte {
	file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDescOnce.Do(func() {
		file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDesc), len(file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDesc)))
	})
	return file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDescData
}

var file_yandex_cloud_iam_v1_access_analyzer_service_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_yandex_cloud_iam_v1_access_analyzer_service_proto_goTypes = []any{
	(*ListSubjectAccessBindingsRequest)(nil),  // 0: yandex.cloud.iam.v1.ListSubjectAccessBindingsRequest
	(*ListSubjectAccessBindingsResponse)(nil), // 1: yandex.cloud.iam.v1.ListSubjectAccessBindingsResponse
	(*SubjectAccessBinding)(nil),              // 2: yandex.cloud.iam.v1.SubjectAccessBinding
	(*Resource)(nil),                          // 3: yandex.cloud.iam.v1.Resource
}
var file_yandex_cloud_iam_v1_access_analyzer_service_proto_depIdxs = []int32{
	2, // 0: yandex.cloud.iam.v1.ListSubjectAccessBindingsResponse.subject_access_bindings:type_name -> yandex.cloud.iam.v1.SubjectAccessBinding
	3, // 1: yandex.cloud.iam.v1.SubjectAccessBinding.resource:type_name -> yandex.cloud.iam.v1.Resource
	0, // 2: yandex.cloud.iam.v1.AccessAnalyzerService.ListSubjectAccessBindings:input_type -> yandex.cloud.iam.v1.ListSubjectAccessBindingsRequest
	1, // 3: yandex.cloud.iam.v1.AccessAnalyzerService.ListSubjectAccessBindings:output_type -> yandex.cloud.iam.v1.ListSubjectAccessBindingsResponse
	3, // [3:4] is the sub-list for method output_type
	2, // [2:3] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_yandex_cloud_iam_v1_access_analyzer_service_proto_init() }
func file_yandex_cloud_iam_v1_access_analyzer_service_proto_init() {
	if File_yandex_cloud_iam_v1_access_analyzer_service_proto != nil {
		return
	}
	file_yandex_cloud_iam_v1_resource_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDesc), len(file_yandex_cloud_iam_v1_access_analyzer_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_yandex_cloud_iam_v1_access_analyzer_service_proto_goTypes,
		DependencyIndexes: file_yandex_cloud_iam_v1_access_analyzer_service_proto_depIdxs,
		MessageInfos:      file_yandex_cloud_iam_v1_access_analyzer_service_proto_msgTypes,
	}.Build()
	File_yandex_cloud_iam_v1_access_analyzer_service_proto = out.File
	file_yandex_cloud_iam_v1_access_analyzer_service_proto_goTypes = nil
	file_yandex_cloud_iam_v1_access_analyzer_service_proto_depIdxs = nil
}
