// Code generated by protoc-gen-goext. DO NOT EDIT.

package dns

import (
	operation "github.com/yandex-cloud/go-genproto/yandex/cloud/operation"
	fieldmaskpb "google.golang.org/protobuf/types/known/fieldmaskpb"
)

func (m *GetDnsFirewallRequest) SetDnsFirewallId(v string) {
	m.DnsFirewallId = v
}

func (m *ListDnsFirewallsRequest) SetFolderId(v string) {
	m.FolderId = v
}

func (m *ListDnsFirewallsRequest) SetPageSize(v int64) {
	m.PageSize = v
}

func (m *ListDnsFirewallsRequest) SetPageToken(v string) {
	m.PageToken = v
}

func (m *ListDnsFirewallsRequest) SetFilter(v string) {
	m.Filter = v
}

func (m *ListDnsFirewallsResponse) SetDnsFirewalls(v []*DnsFirewall) {
	m.DnsFirewalls = v
}

func (m *ListDnsFirewallsResponse) SetNextPageToken(v string) {
	m.NextPageToken = v
}

func (m *CreateDnsFirewallRequest) SetFolderId(v string) {
	m.FolderId = v
}

func (m *CreateDnsFirewallRequest) SetName(v string) {
	m.Name = v
}

func (m *CreateDnsFirewallRequest) SetDescription(v string) {
	m.Description = v
}

func (m *CreateDnsFirewallRequest) SetLabels(v map[string]string) {
	m.Labels = v
}

func (m *CreateDnsFirewallRequest) SetDeletionProtection(v bool) {
	m.DeletionProtection = v
}

func (m *CreateDnsFirewallRequest) SetEnabled(v bool) {
	m.Enabled = v
}

func (m *CreateDnsFirewallRequest) SetResourceConfig(v *DnsFirewall_ResourceConfig) {
	m.ResourceConfig = v
}

func (m *CreateDnsFirewallRequest) SetWhitelistFqdns(v []string) {
	m.WhitelistFqdns = v
}

func (m *CreateDnsFirewallRequest) SetBlacklistFqdns(v []string) {
	m.BlacklistFqdns = v
}

func (m *CreateDnsFirewallMetadata) SetDnsFirewallId(v string) {
	m.DnsFirewallId = v
}

func (m *UpdateDnsFirewallRequest) SetDnsFirewallId(v string) {
	m.DnsFirewallId = v
}

func (m *UpdateDnsFirewallRequest) SetUpdateMask(v *fieldmaskpb.FieldMask) {
	m.UpdateMask = v
}

func (m *UpdateDnsFirewallRequest) SetName(v string) {
	m.Name = v
}

func (m *UpdateDnsFirewallRequest) SetDescription(v string) {
	m.Description = v
}

func (m *UpdateDnsFirewallRequest) SetLabels(v map[string]string) {
	m.Labels = v
}

func (m *UpdateDnsFirewallRequest) SetDeletionProtection(v bool) {
	m.DeletionProtection = v
}

func (m *UpdateDnsFirewallRequest) SetEnabled(v bool) {
	m.Enabled = v
}

func (m *UpdateDnsFirewallRequest) SetResourceConfig(v *DnsFirewall_ResourceConfig) {
	m.ResourceConfig = v
}

func (m *UpdateDnsFirewallRequest) SetWhitelistFqdns(v []string) {
	m.WhitelistFqdns = v
}

func (m *UpdateDnsFirewallRequest) SetBlacklistFqdns(v []string) {
	m.BlacklistFqdns = v
}

func (m *UpdateDnsFirewallMetadata) SetDnsFirewallId(v string) {
	m.DnsFirewallId = v
}

func (m *MoveDnsFirewallRequest) SetDnsFirewallId(v string) {
	m.DnsFirewallId = v
}

func (m *MoveDnsFirewallRequest) SetDestinationFolderId(v string) {
	m.DestinationFolderId = v
}

func (m *MoveDnsFirewallMetadata) SetDnsFirewallId(v string) {
	m.DnsFirewallId = v
}

func (m *DeleteDnsFirewallRequest) SetDnsFirewallId(v string) {
	m.DnsFirewallId = v
}

func (m *DeleteDnsFirewallMetadata) SetDnsFirewallId(v string) {
	m.DnsFirewallId = v
}

func (m *ListDnsFirewallOperationsRequest) SetDnsFirewallId(v string) {
	m.DnsFirewallId = v
}

func (m *ListDnsFirewallOperationsRequest) SetPageSize(v int64) {
	m.PageSize = v
}

func (m *ListDnsFirewallOperationsRequest) SetPageToken(v string) {
	m.PageToken = v
}

func (m *ListDnsFirewallOperationsRequest) SetFilter(v string) {
	m.Filter = v
}

func (m *ListDnsFirewallOperationsResponse) SetOperations(v []*operation.Operation) {
	m.Operations = v
}

func (m *ListDnsFirewallOperationsResponse) SetNextPageToken(v string) {
	m.NextPageToken = v
}
