package bunny

// VideoLibraryService communicates with the /videolibrary API endpoint.
//
// Bunny.net API docs: https://docs.bunny.net/reference/videolibrarypublic_index
type VideoLibraryService struct {
	client *Client
}
