// Code generated by protoc-gen-goext. DO NOT EDIT.

package iam

import (
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
)

func (m *RefreshToken) SetId(v string) {
	m.Id = v
}

func (m *RefreshToken) SetClientInstanceInfo(v string) {
	m.ClientInstanceInfo = v
}

func (m *RefreshToken) SetClientId(v string) {
	m.ClientId = v
}

func (m *RefreshToken) SetSubjectId(v string) {
	m.SubjectId = v
}

func (m *RefreshToken) SetCreatedAt(v *timestamppb.Timestamp) {
	m.CreatedAt = v
}

func (m *RefreshToken) SetExpiresAt(v *timestamppb.Timestamp) {
	m.ExpiresAt = v
}

func (m *RefreshToken) SetLastUsedAt(v *timestamppb.Timestamp) {
	m.LastUsedAt = v
}

func (m *RefreshToken) SetProtectionLevel(v RefreshToken_ProtectionLevel) {
	m.ProtectionLevel = v
}
