// Code generated by protoc-gen-goext. DO NOT EDIT.

package iam

import (
	fieldmaskpb "google.golang.org/protobuf/types/known/fieldmaskpb"
)

func (m *ListOAuthClientsRequest) SetPageSize(v int64) {
	m.PageSize = v
}

func (m *ListOAuthClientsRequest) SetPageToken(v string) {
	m.PageToken = v
}

func (m *ListOAuthClientsRequest) SetFolderId(v string) {
	m.FolderId = v
}

func (m *ListOAuthClientsResponse) SetOauthClients(v []*OAuthClientListView) {
	m.OauthClients = v
}

func (m *ListOAuthClientsResponse) SetNextPageToken(v string) {
	m.NextPageToken = v
}

func (m *GetOAuthClientRequest) SetOauthClientId(v string) {
	m.OauthClientId = v
}

func (m *OAuthClientListView) SetId(v string) {
	m.Id = v
}

func (m *OAuthClientListView) SetName(v string) {
	m.Name = v
}

func (m *CreateOAuthClientMetadata) SetOauthClientId(v string) {
	m.OauthClientId = v
}

func (m *UpdateOAuthClientMetadata) SetOauthClientId(v string) {
	m.OauthClientId = v
}

func (m *DeleteOAuthClientMetadata) SetOauthClientId(v string) {
	m.OauthClientId = v
}

func (m *CreateOAuthClientRequest) SetName(v string) {
	m.Name = v
}

func (m *CreateOAuthClientRequest) SetRedirectUris(v []string) {
	m.RedirectUris = v
}

func (m *CreateOAuthClientRequest) SetScopes(v []string) {
	m.Scopes = v
}

func (m *CreateOAuthClientRequest) SetFolderId(v string) {
	m.FolderId = v
}

func (m *UpdateOAuthClientRequest) SetOauthClientId(v string) {
	m.OauthClientId = v
}

func (m *UpdateOAuthClientRequest) SetUpdateMask(v *fieldmaskpb.FieldMask) {
	m.UpdateMask = v
}

func (m *UpdateOAuthClientRequest) SetName(v string) {
	m.Name = v
}

func (m *UpdateOAuthClientRequest) SetRedirectUris(v []string) {
	m.RedirectUris = v
}

func (m *UpdateOAuthClientRequest) SetScopes(v []string) {
	m.Scopes = v
}

func (m *DeleteOAuthClientRequest) SetOauthClientId(v string) {
	m.OauthClientId = v
}
