package record

const (
	A         = "A (1)"
	NS        = "NS (2)"
	CNAME     = "CNAME (5)"
	SOA       = "SOA (6)"
	PTR       = "PTR (12)"
	HINFO     = "HINFO (13)"
	MX        = "MX (15)"
	TXT       = "TXT (16)"
	RP        = "RP (17)"
	AAAA      = "AAAA (28)"
	SRV       = "SRV (33)"
	NAPTR     = "NAPTR (35)"
	DS        = "DS (43)"
	SSHFP     = "SSHFP (44)"
	TLSA      = "TLSA (52)"
	CDS       = "CDS (59)"
	CDNSKEY   = "CDNSKEY (60)"
	SVCB      = "SVCB (64)"
	HTTPS     = "HTTPS (65)"
	SPF       = "SPF (99)"
	CAA       = "CAA (257)"
	APEXALIAS = "APEXALIAS (65282)"
)
