// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: yandex/cloud/iam/v1/key_service.proto

package iam

import (
	_ "github.com/yandex-cloud/go-genproto/yandex/cloud"
	_ "github.com/yandex-cloud/go-genproto/yandex/cloud/api"
	operation "github.com/yandex-cloud/go-genproto/yandex/cloud/operation"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	fieldmaskpb "google.golang.org/protobuf/types/known/fieldmaskpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type KeyFormat int32

const (
	// Privacy-Enhanced Mail (PEM) format. Default value.
	KeyFormat_PEM_FILE KeyFormat = 0
)

// Enum value maps for KeyFormat.
var (
	KeyFormat_name = map[int32]string{
		0: "PEM_FILE",
	}
	KeyFormat_value = map[string]int32{
		"PEM_FILE": 0,
	}
)

func (x KeyFormat) Enum() *KeyFormat {
	p := new(KeyFormat)
	*p = x
	return p
}

func (x KeyFormat) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (KeyFormat) Descriptor() protoreflect.EnumDescriptor {
	return file_yandex_cloud_iam_v1_key_service_proto_enumTypes[0].Descriptor()
}

func (KeyFormat) Type() protoreflect.EnumType {
	return &file_yandex_cloud_iam_v1_key_service_proto_enumTypes[0]
}

func (x KeyFormat) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use KeyFormat.Descriptor instead.
func (KeyFormat) EnumDescriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_key_service_proto_rawDescGZIP(), []int{0}
}

type GetKeyRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the Key resource to return.
	// To get the ID use a [KeyService.List] request.
	KeyId string `protobuf:"bytes,1,opt,name=key_id,json=keyId,proto3" json:"key_id,omitempty"`
	// Output format of the key.
	Format        KeyFormat `protobuf:"varint,2,opt,name=format,proto3,enum=yandex.cloud.iam.v1.KeyFormat" json:"format,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetKeyRequest) Reset() {
	*x = GetKeyRequest{}
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetKeyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetKeyRequest) ProtoMessage() {}

func (x *GetKeyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetKeyRequest.ProtoReflect.Descriptor instead.
func (*GetKeyRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_key_service_proto_rawDescGZIP(), []int{0}
}

func (x *GetKeyRequest) GetKeyId() string {
	if x != nil {
		return x.KeyId
	}
	return ""
}

func (x *GetKeyRequest) GetFormat() KeyFormat {
	if x != nil {
		return x.Format
	}
	return KeyFormat_PEM_FILE
}

type ListKeysRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Output format of the key.
	Format KeyFormat `protobuf:"varint,1,opt,name=format,proto3,enum=yandex.cloud.iam.v1.KeyFormat" json:"format,omitempty"`
	// ID of the service account to list key pairs for.
	// To get the service account ID, use a [yandex.cloud.iam.v1.ServiceAccountService.List] request.
	// If not specified, it defaults to the subject that made the request.
	ServiceAccountId string `protobuf:"bytes,2,opt,name=service_account_id,json=serviceAccountId,proto3" json:"service_account_id,omitempty"`
	// The maximum number of results per page to return. If the number of available
	// results is larger than [page_size],
	// the service returns a [ListKeysResponse.next_page_token]
	// that can be used to get the next page of results in subsequent list requests.
	// Default value: 100.
	PageSize int64 `protobuf:"varint,3,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Page token. To get the next page of results, set [page_token] to the
	// [ListKeysResponse.next_page_token] returned by a previous list request.
	PageToken     string `protobuf:"bytes,4,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListKeysRequest) Reset() {
	*x = ListKeysRequest{}
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListKeysRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListKeysRequest) ProtoMessage() {}

func (x *ListKeysRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListKeysRequest.ProtoReflect.Descriptor instead.
func (*ListKeysRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_key_service_proto_rawDescGZIP(), []int{1}
}

func (x *ListKeysRequest) GetFormat() KeyFormat {
	if x != nil {
		return x.Format
	}
	return KeyFormat_PEM_FILE
}

func (x *ListKeysRequest) GetServiceAccountId() string {
	if x != nil {
		return x.ServiceAccountId
	}
	return ""
}

func (x *ListKeysRequest) GetPageSize() int64 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListKeysRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

type ListKeysResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// List of Key resources.
	Keys []*Key `protobuf:"bytes,1,rep,name=keys,proto3" json:"keys,omitempty"`
	// This token allows you to get the next page of results for list requests. If the number of results
	// is larger than [ListKeysRequest.page_size], use
	// the [next_page_token] as the value
	// for the [ListKeysRequest.page_token] query parameter
	// in the next list request. Each subsequent list request will have its own
	// [next_page_token] to continue paging through the results.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListKeysResponse) Reset() {
	*x = ListKeysResponse{}
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListKeysResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListKeysResponse) ProtoMessage() {}

func (x *ListKeysResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListKeysResponse.ProtoReflect.Descriptor instead.
func (*ListKeysResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_key_service_proto_rawDescGZIP(), []int{2}
}

func (x *ListKeysResponse) GetKeys() []*Key {
	if x != nil {
		return x.Keys
	}
	return nil
}

func (x *ListKeysResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

type CreateKeyRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the service account to create a key pair for.
	// To get the service account ID, use a [yandex.cloud.iam.v1.ServiceAccountService.List] request.
	// If not specified, it defaults to the subject that made the request.
	ServiceAccountId string `protobuf:"bytes,1,opt,name=service_account_id,json=serviceAccountId,proto3" json:"service_account_id,omitempty"`
	// Description of the key pair.
	Description string `protobuf:"bytes,2,opt,name=description,proto3" json:"description,omitempty"`
	// Output format of the key.
	Format KeyFormat `protobuf:"varint,3,opt,name=format,proto3,enum=yandex.cloud.iam.v1.KeyFormat" json:"format,omitempty"`
	// An algorithm used to generate a key pair of the Key resource.
	KeyAlgorithm  Key_Algorithm `protobuf:"varint,4,opt,name=key_algorithm,json=keyAlgorithm,proto3,enum=yandex.cloud.iam.v1.Key_Algorithm" json:"key_algorithm,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateKeyRequest) Reset() {
	*x = CreateKeyRequest{}
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateKeyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateKeyRequest) ProtoMessage() {}

func (x *CreateKeyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateKeyRequest.ProtoReflect.Descriptor instead.
func (*CreateKeyRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_key_service_proto_rawDescGZIP(), []int{3}
}

func (x *CreateKeyRequest) GetServiceAccountId() string {
	if x != nil {
		return x.ServiceAccountId
	}
	return ""
}

func (x *CreateKeyRequest) GetDescription() string {
	if x != nil {
		return x.Description
	}
	return ""
}

func (x *CreateKeyRequest) GetFormat() KeyFormat {
	if x != nil {
		return x.Format
	}
	return KeyFormat_PEM_FILE
}

func (x *CreateKeyRequest) GetKeyAlgorithm() Key_Algorithm {
	if x != nil {
		return x.KeyAlgorithm
	}
	return Key_ALGORITHM_UNSPECIFIED
}

type CreateKeyResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Key resource.
	Key *Key `protobuf:"bytes,1,opt,name=key,proto3" json:"key,omitempty"`
	// A private key of the Key resource.
	// This key must be stored securely.
	PrivateKey    string `protobuf:"bytes,2,opt,name=private_key,json=privateKey,proto3" json:"private_key,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateKeyResponse) Reset() {
	*x = CreateKeyResponse{}
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateKeyResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateKeyResponse) ProtoMessage() {}

func (x *CreateKeyResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateKeyResponse.ProtoReflect.Descriptor instead.
func (*CreateKeyResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_key_service_proto_rawDescGZIP(), []int{4}
}

func (x *CreateKeyResponse) GetKey() *Key {
	if x != nil {
		return x.Key
	}
	return nil
}

func (x *CreateKeyResponse) GetPrivateKey() string {
	if x != nil {
		return x.PrivateKey
	}
	return ""
}

type UpdateKeyRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the Key resource to update.
	// To get key pair ID, use a [KeyService.List] request.
	KeyId string `protobuf:"bytes,1,opt,name=key_id,json=keyId,proto3" json:"key_id,omitempty"`
	// Field mask that specifies which fields of the Key resource are going to be updated.
	UpdateMask *fieldmaskpb.FieldMask `protobuf:"bytes,2,opt,name=update_mask,json=updateMask,proto3" json:"update_mask,omitempty"`
	// Description of the key pair.
	Description   string `protobuf:"bytes,3,opt,name=description,proto3" json:"description,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateKeyRequest) Reset() {
	*x = UpdateKeyRequest{}
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateKeyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateKeyRequest) ProtoMessage() {}

func (x *UpdateKeyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateKeyRequest.ProtoReflect.Descriptor instead.
func (*UpdateKeyRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_key_service_proto_rawDescGZIP(), []int{5}
}

func (x *UpdateKeyRequest) GetKeyId() string {
	if x != nil {
		return x.KeyId
	}
	return ""
}

func (x *UpdateKeyRequest) GetUpdateMask() *fieldmaskpb.FieldMask {
	if x != nil {
		return x.UpdateMask
	}
	return nil
}

func (x *UpdateKeyRequest) GetDescription() string {
	if x != nil {
		return x.Description
	}
	return ""
}

type UpdateKeyMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the Key resource that is being updated.
	KeyId         string `protobuf:"bytes,1,opt,name=key_id,json=keyId,proto3" json:"key_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateKeyMetadata) Reset() {
	*x = UpdateKeyMetadata{}
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateKeyMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateKeyMetadata) ProtoMessage() {}

func (x *UpdateKeyMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateKeyMetadata.ProtoReflect.Descriptor instead.
func (*UpdateKeyMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_key_service_proto_rawDescGZIP(), []int{6}
}

func (x *UpdateKeyMetadata) GetKeyId() string {
	if x != nil {
		return x.KeyId
	}
	return ""
}

type DeleteKeyRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the key to delete.
	// To get key ID use a [KeyService.List] request.
	KeyId         string `protobuf:"bytes,1,opt,name=key_id,json=keyId,proto3" json:"key_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DeleteKeyRequest) Reset() {
	*x = DeleteKeyRequest{}
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DeleteKeyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DeleteKeyRequest) ProtoMessage() {}

func (x *DeleteKeyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DeleteKeyRequest.ProtoReflect.Descriptor instead.
func (*DeleteKeyRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_key_service_proto_rawDescGZIP(), []int{7}
}

func (x *DeleteKeyRequest) GetKeyId() string {
	if x != nil {
		return x.KeyId
	}
	return ""
}

type DeleteKeyMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the key that is being deleted.
	KeyId         string `protobuf:"bytes,1,opt,name=key_id,json=keyId,proto3" json:"key_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DeleteKeyMetadata) Reset() {
	*x = DeleteKeyMetadata{}
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DeleteKeyMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DeleteKeyMetadata) ProtoMessage() {}

func (x *DeleteKeyMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DeleteKeyMetadata.ProtoReflect.Descriptor instead.
func (*DeleteKeyMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_key_service_proto_rawDescGZIP(), []int{8}
}

func (x *DeleteKeyMetadata) GetKeyId() string {
	if x != nil {
		return x.KeyId
	}
	return ""
}

type ListKeyOperationsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the key to list operations for.
	KeyId string `protobuf:"bytes,1,opt,name=key_id,json=keyId,proto3" json:"key_id,omitempty"`
	// The maximum number of results per page to return. If the number of available
	// results is larger than [page_size],
	// the service returns a [ListKeyOperationsResponse.next_page_token]
	// that can be used to get the next page of results in subsequent list requests.
	// Default value: 100.
	PageSize int64 `protobuf:"varint,2,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Page token. To get the next page of results, set [page_token] to the
	// [ListKeyOperationsResponse.next_page_token] returned by a previous list request.
	PageToken     string `protobuf:"bytes,3,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListKeyOperationsRequest) Reset() {
	*x = ListKeyOperationsRequest{}
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListKeyOperationsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListKeyOperationsRequest) ProtoMessage() {}

func (x *ListKeyOperationsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListKeyOperationsRequest.ProtoReflect.Descriptor instead.
func (*ListKeyOperationsRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_key_service_proto_rawDescGZIP(), []int{9}
}

func (x *ListKeyOperationsRequest) GetKeyId() string {
	if x != nil {
		return x.KeyId
	}
	return ""
}

func (x *ListKeyOperationsRequest) GetPageSize() int64 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListKeyOperationsRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

type ListKeyOperationsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// List of operations for the specified key.
	Operations []*operation.Operation `protobuf:"bytes,1,rep,name=operations,proto3" json:"operations,omitempty"`
	// This token allows you to get the next page of results for list requests. If the number of results
	// is larger than [ListKeyOperationsRequest.page_size], use the [next_page_token] as the value
	// for the [ListKeyOperationsRequest.page_token] query parameter in the next list request.
	// Each subsequent list request will have its own [next_page_token] to continue paging through the results.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListKeyOperationsResponse) Reset() {
	*x = ListKeyOperationsResponse{}
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListKeyOperationsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListKeyOperationsResponse) ProtoMessage() {}

func (x *ListKeyOperationsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_key_service_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListKeyOperationsResponse.ProtoReflect.Descriptor instead.
func (*ListKeyOperationsResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_key_service_proto_rawDescGZIP(), []int{10}
}

func (x *ListKeyOperationsResponse) GetOperations() []*operation.Operation {
	if x != nil {
		return x.Operations
	}
	return nil
}

func (x *ListKeyOperationsResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

var File_yandex_cloud_iam_v1_key_service_proto protoreflect.FileDescriptor

const file_yandex_cloud_iam_v1_key_service_proto_rawDesc = "" +
	"\n" +
	"%yandex/cloud/iam/v1/key_service.proto\x12\x13yandex.cloud.iam.v1\x1a\x1cgoogle/api/annotations.proto\x1a google/protobuf/field_mask.proto\x1a yandex/cloud/api/operation.proto\x1a\x1dyandex/cloud/iam/v1/key.proto\x1a&yandex/cloud/operation/operation.proto\x1a\x1dyandex/cloud/validation.proto\"l\n" +
	"\rGetKeyRequest\x12#\n" +
	"\x06key_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x05keyId\x126\n" +
	"\x06format\x18\x02 \x01(\x0e2\x1e.yandex.cloud.iam.v1.KeyFormatR\x06format\"\xd5\x01\n" +
	"\x0fListKeysRequest\x126\n" +
	"\x06format\x18\x01 \x01(\x0e2\x1e.yandex.cloud.iam.v1.KeyFormatR\x06format\x126\n" +
	"\x12service_account_id\x18\x02 \x01(\tB\b\x8a\xc81\x04<=50R\x10serviceAccountId\x12'\n" +
	"\tpage_size\x18\x03 \x01(\x03B\n" +
	"\xfa\xc71\x060-1000R\bpageSize\x12)\n" +
	"\n" +
	"page_token\x18\x04 \x01(\tB\n" +
	"\x8a\xc81\x06<=2000R\tpageToken\"h\n" +
	"\x10ListKeysResponse\x12,\n" +
	"\x04keys\x18\x01 \x03(\v2\x18.yandex.cloud.iam.v1.KeyR\x04keys\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\xf8\x01\n" +
	"\x10CreateKeyRequest\x126\n" +
	"\x12service_account_id\x18\x01 \x01(\tB\b\x8a\xc81\x04<=50R\x10serviceAccountId\x12+\n" +
	"\vdescription\x18\x02 \x01(\tB\t\x8a\xc81\x05<=256R\vdescription\x126\n" +
	"\x06format\x18\x03 \x01(\x0e2\x1e.yandex.cloud.iam.v1.KeyFormatR\x06format\x12G\n" +
	"\rkey_algorithm\x18\x04 \x01(\x0e2\".yandex.cloud.iam.v1.Key.AlgorithmR\fkeyAlgorithm\"`\n" +
	"\x11CreateKeyResponse\x12*\n" +
	"\x03key\x18\x01 \x01(\v2\x18.yandex.cloud.iam.v1.KeyR\x03key\x12\x1f\n" +
	"\vprivate_key\x18\x02 \x01(\tR\n" +
	"privateKey\"\xa1\x01\n" +
	"\x10UpdateKeyRequest\x12#\n" +
	"\x06key_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x05keyId\x12;\n" +
	"\vupdate_mask\x18\x02 \x01(\v2\x1a.google.protobuf.FieldMaskR\n" +
	"updateMask\x12+\n" +
	"\vdescription\x18\x03 \x01(\tB\t\x8a\xc81\x05<=256R\vdescription\"*\n" +
	"\x11UpdateKeyMetadata\x12\x15\n" +
	"\x06key_id\x18\x01 \x01(\tR\x05keyId\"7\n" +
	"\x10DeleteKeyRequest\x12#\n" +
	"\x06key_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x05keyId\"*\n" +
	"\x11DeleteKeyMetadata\x12\x15\n" +
	"\x06key_id\x18\x01 \x01(\tR\x05keyId\"\x93\x01\n" +
	"\x18ListKeyOperationsRequest\x12#\n" +
	"\x06key_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x05keyId\x12'\n" +
	"\tpage_size\x18\x02 \x01(\x03B\n" +
	"\xfa\xc71\x060-1000R\bpageSize\x12)\n" +
	"\n" +
	"page_token\x18\x03 \x01(\tB\n" +
	"\x8a\xc81\x06<=2000R\tpageToken\"\x86\x01\n" +
	"\x19ListKeyOperationsResponse\x12A\n" +
	"\n" +
	"operations\x18\x01 \x03(\v2!.yandex.cloud.operation.OperationR\n" +
	"operations\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken*\x19\n" +
	"\tKeyFormat\x12\f\n" +
	"\bPEM_FILE\x10\x002\x9e\x06\n" +
	"\n" +
	"KeyService\x12b\n" +
	"\x03Get\x12\".yandex.cloud.iam.v1.GetKeyRequest\x1a\x18.yandex.cloud.iam.v1.Key\"\x1d\x82\xd3\xe4\x93\x02\x17\x12\x15/iam/v1/keys/{key_id}\x12i\n" +
	"\x04List\x12$.yandex.cloud.iam.v1.ListKeysRequest\x1a%.yandex.cloud.iam.v1.ListKeysResponse\"\x14\x82\xd3\xe4\x93\x02\x0e\x12\f/iam/v1/keys\x12p\n" +
	"\x06Create\x12%.yandex.cloud.iam.v1.CreateKeyRequest\x1a&.yandex.cloud.iam.v1.CreateKeyResponse\"\x17\x82\xd3\xe4\x93\x02\x11:\x01*\"\f/iam/v1/keys\x12\x90\x01\n" +
	"\x06Update\x12%.yandex.cloud.iam.v1.UpdateKeyRequest\x1a!.yandex.cloud.operation.Operation\"<\xb2\xd2*\x18\n" +
	"\x11UpdateKeyMetadata\x12\x03Key\x82\xd3\xe4\x93\x02\x1a:\x01*2\x15/iam/v1/keys/{key_id}\x12\x9f\x01\n" +
	"\x06Delete\x12%.yandex.cloud.iam.v1.DeleteKeyRequest\x1a!.yandex.cloud.operation.Operation\"K\xb2\xd2**\n" +
	"\x11DeleteKeyMetadata\x12\x15google.protobuf.Empty\x82\xd3\xe4\x93\x02\x17*\x15/iam/v1/keys/{key_id}\x12\x99\x01\n" +
	"\x0eListOperations\x12-.yandex.cloud.iam.v1.ListKeyOperationsRequest\x1a..yandex.cloud.iam.v1.ListKeyOperationsResponse\"(\x82\xd3\xe4\x93\x02\"\x12 /iam/v1/keys/{key_id}/operationsBV\n" +
	"\x17yandex.cloud.api.iam.v1Z;github.com/yandex-cloud/go-genproto/yandex/cloud/iam/v1;iamb\x06proto3"

var (
	file_yandex_cloud_iam_v1_key_service_proto_rawDescOnce sync.Once
	file_yandex_cloud_iam_v1_key_service_proto_rawDescData []byte
)

func file_yandex_cloud_iam_v1_key_service_proto_rawDescGZIP() []byte {
	file_yandex_cloud_iam_v1_key_service_proto_rawDescOnce.Do(func() {
		file_yandex_cloud_iam_v1_key_service_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_key_service_proto_rawDesc), len(file_yandex_cloud_iam_v1_key_service_proto_rawDesc)))
	})
	return file_yandex_cloud_iam_v1_key_service_proto_rawDescData
}

var file_yandex_cloud_iam_v1_key_service_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_yandex_cloud_iam_v1_key_service_proto_msgTypes = make([]protoimpl.MessageInfo, 11)
var file_yandex_cloud_iam_v1_key_service_proto_goTypes = []any{
	(KeyFormat)(0),                    // 0: yandex.cloud.iam.v1.KeyFormat
	(*GetKeyRequest)(nil),             // 1: yandex.cloud.iam.v1.GetKeyRequest
	(*ListKeysRequest)(nil),           // 2: yandex.cloud.iam.v1.ListKeysRequest
	(*ListKeysResponse)(nil),          // 3: yandex.cloud.iam.v1.ListKeysResponse
	(*CreateKeyRequest)(nil),          // 4: yandex.cloud.iam.v1.CreateKeyRequest
	(*CreateKeyResponse)(nil),         // 5: yandex.cloud.iam.v1.CreateKeyResponse
	(*UpdateKeyRequest)(nil),          // 6: yandex.cloud.iam.v1.UpdateKeyRequest
	(*UpdateKeyMetadata)(nil),         // 7: yandex.cloud.iam.v1.UpdateKeyMetadata
	(*DeleteKeyRequest)(nil),          // 8: yandex.cloud.iam.v1.DeleteKeyRequest
	(*DeleteKeyMetadata)(nil),         // 9: yandex.cloud.iam.v1.DeleteKeyMetadata
	(*ListKeyOperationsRequest)(nil),  // 10: yandex.cloud.iam.v1.ListKeyOperationsRequest
	(*ListKeyOperationsResponse)(nil), // 11: yandex.cloud.iam.v1.ListKeyOperationsResponse
	(*Key)(nil),                       // 12: yandex.cloud.iam.v1.Key
	(Key_Algorithm)(0),                // 13: yandex.cloud.iam.v1.Key.Algorithm
	(*fieldmaskpb.FieldMask)(nil),     // 14: google.protobuf.FieldMask
	(*operation.Operation)(nil),       // 15: yandex.cloud.operation.Operation
}
var file_yandex_cloud_iam_v1_key_service_proto_depIdxs = []int32{
	0,  // 0: yandex.cloud.iam.v1.GetKeyRequest.format:type_name -> yandex.cloud.iam.v1.KeyFormat
	0,  // 1: yandex.cloud.iam.v1.ListKeysRequest.format:type_name -> yandex.cloud.iam.v1.KeyFormat
	12, // 2: yandex.cloud.iam.v1.ListKeysResponse.keys:type_name -> yandex.cloud.iam.v1.Key
	0,  // 3: yandex.cloud.iam.v1.CreateKeyRequest.format:type_name -> yandex.cloud.iam.v1.KeyFormat
	13, // 4: yandex.cloud.iam.v1.CreateKeyRequest.key_algorithm:type_name -> yandex.cloud.iam.v1.Key.Algorithm
	12, // 5: yandex.cloud.iam.v1.CreateKeyResponse.key:type_name -> yandex.cloud.iam.v1.Key
	14, // 6: yandex.cloud.iam.v1.UpdateKeyRequest.update_mask:type_name -> google.protobuf.FieldMask
	15, // 7: yandex.cloud.iam.v1.ListKeyOperationsResponse.operations:type_name -> yandex.cloud.operation.Operation
	1,  // 8: yandex.cloud.iam.v1.KeyService.Get:input_type -> yandex.cloud.iam.v1.GetKeyRequest
	2,  // 9: yandex.cloud.iam.v1.KeyService.List:input_type -> yandex.cloud.iam.v1.ListKeysRequest
	4,  // 10: yandex.cloud.iam.v1.KeyService.Create:input_type -> yandex.cloud.iam.v1.CreateKeyRequest
	6,  // 11: yandex.cloud.iam.v1.KeyService.Update:input_type -> yandex.cloud.iam.v1.UpdateKeyRequest
	8,  // 12: yandex.cloud.iam.v1.KeyService.Delete:input_type -> yandex.cloud.iam.v1.DeleteKeyRequest
	10, // 13: yandex.cloud.iam.v1.KeyService.ListOperations:input_type -> yandex.cloud.iam.v1.ListKeyOperationsRequest
	12, // 14: yandex.cloud.iam.v1.KeyService.Get:output_type -> yandex.cloud.iam.v1.Key
	3,  // 15: yandex.cloud.iam.v1.KeyService.List:output_type -> yandex.cloud.iam.v1.ListKeysResponse
	5,  // 16: yandex.cloud.iam.v1.KeyService.Create:output_type -> yandex.cloud.iam.v1.CreateKeyResponse
	15, // 17: yandex.cloud.iam.v1.KeyService.Update:output_type -> yandex.cloud.operation.Operation
	15, // 18: yandex.cloud.iam.v1.KeyService.Delete:output_type -> yandex.cloud.operation.Operation
	11, // 19: yandex.cloud.iam.v1.KeyService.ListOperations:output_type -> yandex.cloud.iam.v1.ListKeyOperationsResponse
	14, // [14:20] is the sub-list for method output_type
	8,  // [8:14] is the sub-list for method input_type
	8,  // [8:8] is the sub-list for extension type_name
	8,  // [8:8] is the sub-list for extension extendee
	0,  // [0:8] is the sub-list for field type_name
}

func init() { file_yandex_cloud_iam_v1_key_service_proto_init() }
func file_yandex_cloud_iam_v1_key_service_proto_init() {
	if File_yandex_cloud_iam_v1_key_service_proto != nil {
		return
	}
	file_yandex_cloud_iam_v1_key_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_key_service_proto_rawDesc), len(file_yandex_cloud_iam_v1_key_service_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   11,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_yandex_cloud_iam_v1_key_service_proto_goTypes,
		DependencyIndexes: file_yandex_cloud_iam_v1_key_service_proto_depIdxs,
		EnumInfos:         file_yandex_cloud_iam_v1_key_service_proto_enumTypes,
		MessageInfos:      file_yandex_cloud_iam_v1_key_service_proto_msgTypes,
	}.Build()
	File_yandex_cloud_iam_v1_key_service_proto = out.File
	file_yandex_cloud_iam_v1_key_service_proto_goTypes = nil
	file_yandex_cloud_iam_v1_key_service_proto_depIdxs = nil
}
