// Code generated by protoc-gen-goext. DO NOT EDIT.

package dns

import (
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
)

func (m *DnsFirewall) SetId(v string) {
	m.Id = v
}

func (m *DnsFirewall) SetFolderId(v string) {
	m.FolderId = v
}

func (m *DnsFirewall) SetCreatedAt(v *timestamppb.Timestamp) {
	m.CreatedAt = v
}

func (m *DnsFirewall) SetName(v string) {
	m.Name = v
}

func (m *DnsFirewall) SetDescription(v string) {
	m.Description = v
}

func (m *DnsFirewall) SetLabels(v map[string]string) {
	m.Labels = v
}

func (m *DnsFirewall) SetDeletionProtection(v bool) {
	m.DeletionProtection = v
}

func (m *DnsFirewall) SetEnabled(v bool) {
	m.Enabled = v
}

func (m *DnsFirewall) SetResourceConfig(v *DnsFirewall_ResourceConfig) {
	m.ResourceConfig = v
}

func (m *DnsFirewall) SetWhitelistFqdns(v []string) {
	m.WhitelistFqdns = v
}

func (m *DnsFirewall) SetBlacklistFqdns(v []string) {
	m.BlacklistFqdns = v
}

func (m *DnsFirewall_ResourceConfig) SetType(v DnsFirewall_ResourceConfig_ResourceType) {
	m.Type = v
}

func (m *DnsFirewall_ResourceConfig) SetResourceIds(v []string) {
	m.ResourceIds = v
}

func (m *DnsFirewall_ResourceConfig) SetLockingPolicyEnabled(v bool) {
	m.LockingPolicyEnabled = v
}
