// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: yandex/cloud/iam/v1/oauth_client_secret_service.proto

package iam

import (
	_ "github.com/yandex-cloud/go-genproto/yandex/cloud"
	_ "github.com/yandex-cloud/go-genproto/yandex/cloud/api"
	operation "github.com/yandex-cloud/go-genproto/yandex/cloud/operation"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type GetOAuthClientSecretRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the OAuthClientSecret resource to return.
	// To get the OAuthClientSecret ID, use a [OAuthClientSecretService.List] request.
	OauthClientSecretId string `protobuf:"bytes,1,opt,name=oauth_client_secret_id,json=oauthClientSecretId,proto3" json:"oauth_client_secret_id,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *GetOAuthClientSecretRequest) Reset() {
	*x = GetOAuthClientSecretRequest{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetOAuthClientSecretRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetOAuthClientSecretRequest) ProtoMessage() {}

func (x *GetOAuthClientSecretRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetOAuthClientSecretRequest.ProtoReflect.Descriptor instead.
func (*GetOAuthClientSecretRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescGZIP(), []int{0}
}

func (x *GetOAuthClientSecretRequest) GetOauthClientSecretId() string {
	if x != nil {
		return x.OauthClientSecretId
	}
	return ""
}

type ListOAuthClientSecretsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The maximum number of results per page to return. If the number of available
	// results is larger than [page_size],
	// the service returns a [ListOAuthClientSecretsResponse.next_page_token]
	// that can be used to get the next page of results in subsequent list requests.
	PageSize int64 `protobuf:"varint,1,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Page token. To get the next page of results, set [page_token] to the
	// [ListOAuthClientSecretsResponse.next_page_token] returned by a previous list request.
	PageToken string `protobuf:"bytes,2,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	// ID of the OAuthClient resource to list OAuthClientSecret resources for.
	// To get the oauth client ID, use a [yandex.cloud.iam.v1.OAuthClientService.List] request.
	OauthClientId string `protobuf:"bytes,3,opt,name=oauth_client_id,json=oauthClientId,proto3" json:"oauth_client_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListOAuthClientSecretsRequest) Reset() {
	*x = ListOAuthClientSecretsRequest{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListOAuthClientSecretsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListOAuthClientSecretsRequest) ProtoMessage() {}

func (x *ListOAuthClientSecretsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListOAuthClientSecretsRequest.ProtoReflect.Descriptor instead.
func (*ListOAuthClientSecretsRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescGZIP(), []int{1}
}

func (x *ListOAuthClientSecretsRequest) GetPageSize() int64 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListOAuthClientSecretsRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

func (x *ListOAuthClientSecretsRequest) GetOauthClientId() string {
	if x != nil {
		return x.OauthClientId
	}
	return ""
}

type ListOAuthClientSecretsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// List of OAuthClientSecret resources of the specified OAuthClient resource
	OauthClientSecrets []*OAuthClientSecret `protobuf:"bytes,1,rep,name=oauth_client_secrets,json=oauthClientSecrets,proto3" json:"oauth_client_secrets,omitempty"`
	// This token allows you to get the next page of results for list requests. If the number of results
	// is larger than [ListOAuthClientSecretsRequest.page_size], use
	// the [next_page_token] as the value
	// for the [ListOAuthClientSecretsRequest.page_token] query parameter
	// in the next list request. Each subsequent list request will have its own
	// [next_page_token] to continue paging through the results.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListOAuthClientSecretsResponse) Reset() {
	*x = ListOAuthClientSecretsResponse{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListOAuthClientSecretsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListOAuthClientSecretsResponse) ProtoMessage() {}

func (x *ListOAuthClientSecretsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListOAuthClientSecretsResponse.ProtoReflect.Descriptor instead.
func (*ListOAuthClientSecretsResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescGZIP(), []int{2}
}

func (x *ListOAuthClientSecretsResponse) GetOauthClientSecrets() []*OAuthClientSecret {
	if x != nil {
		return x.OauthClientSecrets
	}
	return nil
}

func (x *ListOAuthClientSecretsResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

type CreateOAuthClientSecretRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the OAuthClient resource to create OAuthClientSecret resource for.
	// To get the oauth client ID, use a [yandex.cloud.iam.v1.OAuthClientService.List] request.
	OauthClientId string `protobuf:"bytes,1,opt,name=oauth_client_id,json=oauthClientId,proto3" json:"oauth_client_id,omitempty"`
	// Description of the OAuthClientResource.
	Description   string `protobuf:"bytes,2,opt,name=description,proto3" json:"description,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateOAuthClientSecretRequest) Reset() {
	*x = CreateOAuthClientSecretRequest{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateOAuthClientSecretRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateOAuthClientSecretRequest) ProtoMessage() {}

func (x *CreateOAuthClientSecretRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateOAuthClientSecretRequest.ProtoReflect.Descriptor instead.
func (*CreateOAuthClientSecretRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescGZIP(), []int{3}
}

func (x *CreateOAuthClientSecretRequest) GetOauthClientId() string {
	if x != nil {
		return x.OauthClientId
	}
	return ""
}

func (x *CreateOAuthClientSecretRequest) GetDescription() string {
	if x != nil {
		return x.Description
	}
	return ""
}

type CreateOAuthClientSecretMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the OAuthClientSecretResource that is being created.
	OauthClientSecretId string `protobuf:"bytes,1,opt,name=oauth_client_secret_id,json=oauthClientSecretId,proto3" json:"oauth_client_secret_id,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *CreateOAuthClientSecretMetadata) Reset() {
	*x = CreateOAuthClientSecretMetadata{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateOAuthClientSecretMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateOAuthClientSecretMetadata) ProtoMessage() {}

func (x *CreateOAuthClientSecretMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateOAuthClientSecretMetadata.ProtoReflect.Descriptor instead.
func (*CreateOAuthClientSecretMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescGZIP(), []int{4}
}

func (x *CreateOAuthClientSecretMetadata) GetOauthClientSecretId() string {
	if x != nil {
		return x.OauthClientSecretId
	}
	return ""
}

type CreateOAuthClientSecretResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// OAuthClientSecret resource.
	OauthClientSecret *OAuthClientSecret `protobuf:"bytes,1,opt,name=oauth_client_secret,json=oauthClientSecret,proto3" json:"oauth_client_secret,omitempty"`
	// The secret value of OAuthClientSecret resource.
	// This value can be used for lient secret based authentication.
	// This value must be stored securely.
	SecretValue   string `protobuf:"bytes,2,opt,name=secret_value,json=secretValue,proto3" json:"secret_value,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateOAuthClientSecretResponse) Reset() {
	*x = CreateOAuthClientSecretResponse{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateOAuthClientSecretResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateOAuthClientSecretResponse) ProtoMessage() {}

func (x *CreateOAuthClientSecretResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateOAuthClientSecretResponse.ProtoReflect.Descriptor instead.
func (*CreateOAuthClientSecretResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescGZIP(), []int{5}
}

func (x *CreateOAuthClientSecretResponse) GetOauthClientSecret() *OAuthClientSecret {
	if x != nil {
		return x.OauthClientSecret
	}
	return nil
}

func (x *CreateOAuthClientSecretResponse) GetSecretValue() string {
	if x != nil {
		return x.SecretValue
	}
	return ""
}

type DeleteOAuthClientSecretRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the OAuthClientSecret resource to delete.
	OauthClientSecretId string `protobuf:"bytes,1,opt,name=oauth_client_secret_id,json=oauthClientSecretId,proto3" json:"oauth_client_secret_id,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *DeleteOAuthClientSecretRequest) Reset() {
	*x = DeleteOAuthClientSecretRequest{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DeleteOAuthClientSecretRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DeleteOAuthClientSecretRequest) ProtoMessage() {}

func (x *DeleteOAuthClientSecretRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DeleteOAuthClientSecretRequest.ProtoReflect.Descriptor instead.
func (*DeleteOAuthClientSecretRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescGZIP(), []int{6}
}

func (x *DeleteOAuthClientSecretRequest) GetOauthClientSecretId() string {
	if x != nil {
		return x.OauthClientSecretId
	}
	return ""
}

type DeleteOAuthClientSecretMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the OAuthClientSecretResource that is being deleted.
	OauthClientSecretId string `protobuf:"bytes,1,opt,name=oauth_client_secret_id,json=oauthClientSecretId,proto3" json:"oauth_client_secret_id,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *DeleteOAuthClientSecretMetadata) Reset() {
	*x = DeleteOAuthClientSecretMetadata{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DeleteOAuthClientSecretMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DeleteOAuthClientSecretMetadata) ProtoMessage() {}

func (x *DeleteOAuthClientSecretMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DeleteOAuthClientSecretMetadata.ProtoReflect.Descriptor instead.
func (*DeleteOAuthClientSecretMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescGZIP(), []int{7}
}

func (x *DeleteOAuthClientSecretMetadata) GetOauthClientSecretId() string {
	if x != nil {
		return x.OauthClientSecretId
	}
	return ""
}

var File_yandex_cloud_iam_v1_oauth_client_secret_service_proto protoreflect.FileDescriptor

const file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDesc = "" +
	"\n" +
	"5yandex/cloud/iam/v1/oauth_client_secret_service.proto\x12\x13yandex.cloud.iam.v1\x1a\x1cgoogle/api/annotations.proto\x1a yandex/cloud/api/operation.proto\x1a-yandex/cloud/iam/v1/oauth_client_secret.proto\x1a&yandex/cloud/operation/operation.proto\x1a\x1dyandex/cloud/validation.proto\"`\n" +
	"\x1bGetOAuthClientSecretRequest\x12A\n" +
	"\x16oauth_client_secret_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x13oauthClientSecretId\"\xa9\x01\n" +
	"\x1dListOAuthClientSecretsRequest\x12'\n" +
	"\tpage_size\x18\x01 \x01(\x03B\n" +
	"\xfa\xc71\x060-1000R\bpageSize\x12)\n" +
	"\n" +
	"page_token\x18\x02 \x01(\tB\n" +
	"\x8a\xc81\x06<=2000R\tpageToken\x124\n" +
	"\x0foauth_client_id\x18\x03 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\roauthClientId\"\xa2\x01\n" +
	"\x1eListOAuthClientSecretsResponse\x12X\n" +
	"\x14oauth_client_secrets\x18\x01 \x03(\v2&.yandex.cloud.iam.v1.OAuthClientSecretR\x12oauthClientSecrets\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\x83\x01\n" +
	"\x1eCreateOAuthClientSecretRequest\x124\n" +
	"\x0foauth_client_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\roauthClientId\x12+\n" +
	"\vdescription\x18\x02 \x01(\tB\t\x8a\xc81\x05<=256R\vdescription\"V\n" +
	"\x1fCreateOAuthClientSecretMetadata\x123\n" +
	"\x16oauth_client_secret_id\x18\x01 \x01(\tR\x13oauthClientSecretId\"\x9c\x01\n" +
	"\x1fCreateOAuthClientSecretResponse\x12V\n" +
	"\x13oauth_client_secret\x18\x01 \x01(\v2&.yandex.cloud.iam.v1.OAuthClientSecretR\x11oauthClientSecret\x12!\n" +
	"\fsecret_value\x18\x02 \x01(\tR\vsecretValue\"c\n" +
	"\x1eDeleteOAuthClientSecretRequest\x12A\n" +
	"\x16oauth_client_secret_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x13oauthClientSecretId\"V\n" +
	"\x1fDeleteOAuthClientSecretMetadata\x123\n" +
	"\x16oauth_client_secret_id\x18\x01 \x01(\tR\x13oauthClientSecretId2\xfb\x05\n" +
	"\x18OAuthClientSecretService\x12\x9c\x01\n" +
	"\x03Get\x120.yandex.cloud.iam.v1.GetOAuthClientSecretRequest\x1a&.yandex.cloud.iam.v1.OAuthClientSecret\";\x82\xd3\xe4\x93\x025\x123/iam/v1/oauthClientSecrets/{oauth_client_secret_id}\x12\x93\x01\n" +
	"\x04List\x122.yandex.cloud.iam.v1.ListOAuthClientSecretsRequest\x1a3.yandex.cloud.iam.v1.ListOAuthClientSecretsResponse\"\"\x82\xd3\xe4\x93\x02\x1c\x12\x1a/iam/v1/oauthClientSecrets\x12\xcd\x01\n" +
	"\x06Create\x123.yandex.cloud.iam.v1.CreateOAuthClientSecretRequest\x1a!.yandex.cloud.operation.Operation\"k\xb2\xd2*B\n" +
	"\x1fCreateOAuthClientSecretMetadata\x12\x1fCreateOAuthClientSecretResponse\x82\xd3\xe4\x93\x02\x1f:\x01*\"\x1a/iam/v1/oauthClientSecrets\x12\xd9\x01\n" +
	"\x06Delete\x123.yandex.cloud.iam.v1.DeleteOAuthClientSecretRequest\x1a!.yandex.cloud.operation.Operation\"w\xb2\xd2*8\n" +
	"\x1fDeleteOAuthClientSecretMetadata\x12\x15google.protobuf.Empty\x82\xd3\xe4\x93\x025*3/iam/v1/oauthClientSecrets/{oauth_client_secret_id}BV\n" +
	"\x17yandex.cloud.api.iam.v1Z;github.com/yandex-cloud/go-genproto/yandex/cloud/iam/v1;iamb\x06proto3"

var (
	file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescOnce sync.Once
	file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescData []byte
)

func file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescGZIP() []byte {
	file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescOnce.Do(func() {
		file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDesc), len(file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDesc)))
	})
	return file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDescData
}

var file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes = make([]protoimpl.MessageInfo, 8)
var file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_goTypes = []any{
	(*GetOAuthClientSecretRequest)(nil),     // 0: yandex.cloud.iam.v1.GetOAuthClientSecretRequest
	(*ListOAuthClientSecretsRequest)(nil),   // 1: yandex.cloud.iam.v1.ListOAuthClientSecretsRequest
	(*ListOAuthClientSecretsResponse)(nil),  // 2: yandex.cloud.iam.v1.ListOAuthClientSecretsResponse
	(*CreateOAuthClientSecretRequest)(nil),  // 3: yandex.cloud.iam.v1.CreateOAuthClientSecretRequest
	(*CreateOAuthClientSecretMetadata)(nil), // 4: yandex.cloud.iam.v1.CreateOAuthClientSecretMetadata
	(*CreateOAuthClientSecretResponse)(nil), // 5: yandex.cloud.iam.v1.CreateOAuthClientSecretResponse
	(*DeleteOAuthClientSecretRequest)(nil),  // 6: yandex.cloud.iam.v1.DeleteOAuthClientSecretRequest
	(*DeleteOAuthClientSecretMetadata)(nil), // 7: yandex.cloud.iam.v1.DeleteOAuthClientSecretMetadata
	(*OAuthClientSecret)(nil),               // 8: yandex.cloud.iam.v1.OAuthClientSecret
	(*operation.Operation)(nil),             // 9: yandex.cloud.operation.Operation
}
var file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_depIdxs = []int32{
	8, // 0: yandex.cloud.iam.v1.ListOAuthClientSecretsResponse.oauth_client_secrets:type_name -> yandex.cloud.iam.v1.OAuthClientSecret
	8, // 1: yandex.cloud.iam.v1.CreateOAuthClientSecretResponse.oauth_client_secret:type_name -> yandex.cloud.iam.v1.OAuthClientSecret
	0, // 2: yandex.cloud.iam.v1.OAuthClientSecretService.Get:input_type -> yandex.cloud.iam.v1.GetOAuthClientSecretRequest
	1, // 3: yandex.cloud.iam.v1.OAuthClientSecretService.List:input_type -> yandex.cloud.iam.v1.ListOAuthClientSecretsRequest
	3, // 4: yandex.cloud.iam.v1.OAuthClientSecretService.Create:input_type -> yandex.cloud.iam.v1.CreateOAuthClientSecretRequest
	6, // 5: yandex.cloud.iam.v1.OAuthClientSecretService.Delete:input_type -> yandex.cloud.iam.v1.DeleteOAuthClientSecretRequest
	8, // 6: yandex.cloud.iam.v1.OAuthClientSecretService.Get:output_type -> yandex.cloud.iam.v1.OAuthClientSecret
	2, // 7: yandex.cloud.iam.v1.OAuthClientSecretService.List:output_type -> yandex.cloud.iam.v1.ListOAuthClientSecretsResponse
	9, // 8: yandex.cloud.iam.v1.OAuthClientSecretService.Create:output_type -> yandex.cloud.operation.Operation
	9, // 9: yandex.cloud.iam.v1.OAuthClientSecretService.Delete:output_type -> yandex.cloud.operation.Operation
	6, // [6:10] is the sub-list for method output_type
	2, // [2:6] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_init() }
func file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_init() {
	if File_yandex_cloud_iam_v1_oauth_client_secret_service_proto != nil {
		return
	}
	file_yandex_cloud_iam_v1_oauth_client_secret_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDesc), len(file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   8,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_goTypes,
		DependencyIndexes: file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_depIdxs,
		MessageInfos:      file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_msgTypes,
	}.Build()
	File_yandex_cloud_iam_v1_oauth_client_secret_service_proto = out.File
	file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_goTypes = nil
	file_yandex_cloud_iam_v1_oauth_client_secret_service_proto_depIdxs = nil
}
