// Code generated by protoc-gen-goext. DO NOT EDIT.

package access

func (m *Subject) SetId(v string) {
	m.Id = v
}

func (m *Subject) SetType(v string) {
	m.Type = v
}

func (m *AccessBinding) SetRoleId(v string) {
	m.RoleId = v
}

func (m *AccessBinding) SetSubject(v *Subject) {
	m.Subject = v
}

func (m *AccessPolicy) SetId(v string) {
	m.Id = v
}

func (m *AccessPolicy) SetName(v string) {
	m.Name = v
}

func (m *AccessPolicy) SetDescription(v string) {
	m.Description = v
}

func (m *AccessPolicyBinding) SetAccessPolicyTemplateId(v string) {
	m.AccessPolicyTemplateId = v
}

func (m *AccessPolicyBinding) SetParameters(v map[string]string) {
	m.Parameters = v
}

func (m *BindAccessPolicyRequest) SetResourceId(v string) {
	m.ResourceId = v
}

func (m *BindAccessPolicyRequest) SetAccessPolicyBinding(v *AccessPolicyBinding) {
	m.AccessPolicyBinding = v
}

func (m *BindAccessPolicyMetadata) SetResourceId(v string) {
	m.ResourceId = v
}

func (m *BindAccessPolicyMetadata) SetAccessPolicyBinding(v *AccessPolicyBinding) {
	m.AccessPolicyBinding = v
}

func (m *ListAccessBindingsRequest) SetResourceId(v string) {
	m.ResourceId = v
}

func (m *ListAccessBindingsRequest) SetPageSize(v int64) {
	m.PageSize = v
}

func (m *ListAccessBindingsRequest) SetPageToken(v string) {
	m.PageToken = v
}

func (m *ListAccessBindingsResponse) SetAccessBindings(v []*AccessBinding) {
	m.AccessBindings = v
}

func (m *ListAccessBindingsResponse) SetNextPageToken(v string) {
	m.NextPageToken = v
}

func (m *ListAccessPolicyBindingsRequest) SetResourceId(v string) {
	m.ResourceId = v
}

func (m *ListAccessPolicyBindingsRequest) SetPageSize(v int64) {
	m.PageSize = v
}

func (m *ListAccessPolicyBindingsRequest) SetPageToken(v string) {
	m.PageToken = v
}

func (m *ListAccessPolicyBindingsResponse) SetAccessPolicyBindings(v []*AccessPolicyBinding) {
	m.AccessPolicyBindings = v
}

func (m *ListAccessPolicyBindingsResponse) SetNextPageToken(v string) {
	m.NextPageToken = v
}

func (m *UnbindAccessPolicyRequest) SetResourceId(v string) {
	m.ResourceId = v
}

func (m *UnbindAccessPolicyRequest) SetAccessPolicyTemplateId(v string) {
	m.AccessPolicyTemplateId = v
}

func (m *UnbindAccessPolicyMetadata) SetResourceId(v string) {
	m.ResourceId = v
}

func (m *UnbindAccessPolicyMetadata) SetAccessPolicyTemplateId(v string) {
	m.AccessPolicyTemplateId = v
}

func (m *UpdateAccessPolicyBindingParametersRequest) SetResourceId(v string) {
	m.ResourceId = v
}

func (m *UpdateAccessPolicyBindingParametersRequest) SetAccessPolicyBinding(v *AccessPolicyBinding) {
	m.AccessPolicyBinding = v
}

func (m *UpdateAccessPolicyBindingParametersMetadata) SetResourceId(v string) {
	m.ResourceId = v
}

func (m *UpdateAccessPolicyBindingParametersMetadata) SetAccessPolicyBinding(v *AccessPolicyBinding) {
	m.AccessPolicyBinding = v
}

func (m *SetAccessBindingsRequest) SetResourceId(v string) {
	m.ResourceId = v
}

func (m *SetAccessBindingsRequest) SetAccessBindings(v []*AccessBinding) {
	m.AccessBindings = v
}

func (m *SetAccessBindingsMetadata) SetResourceId(v string) {
	m.ResourceId = v
}

func (m *UpdateAccessBindingsRequest) SetResourceId(v string) {
	m.ResourceId = v
}

func (m *UpdateAccessBindingsRequest) SetAccessBindingDeltas(v []*AccessBindingDelta) {
	m.AccessBindingDeltas = v
}

func (m *UpdateAccessBindingsMetadata) SetResourceId(v string) {
	m.ResourceId = v
}

func (m *AccessBindingDelta) SetAction(v AccessBindingAction) {
	m.Action = v
}

func (m *AccessBindingDelta) SetAccessBinding(v *AccessBinding) {
	m.AccessBinding = v
}

func (m *AccessBindingsOperationResult) SetEffectiveDeltas(v []*AccessBindingDelta) {
	m.EffectiveDeltas = v
}
