// Copyright 2025- The sacloud/saclient-go Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package saclient

// Miscellaneous routines

// Ptr returns pointer of the value.
// Handy when you need to pass pointer of literal value.
//
// ```golang
//
//	import "github.com/sacloud/saclient-go"
//
//	func main() {
//		ret, err := OtherFunc(saclient.Ptr("string literal"))
//	}
//
// ```
func Ptr[T any](t T) *T { return &t }
