// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: yandex/cloud/api/operation.proto

package api

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	descriptorpb "google.golang.org/protobuf/types/descriptorpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Operation is annotation for rpc that returns longrunning operation, describes
// message types that will be returned in metadata [google.protobuf.Any], and
// in response [google.protobuf.Any] (for successful operation).
type Operation struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Optional. If present, rpc returns operation which metadata field will
	// contains message of specified type.
	Metadata string `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"` // Optional.
	// Required. rpc returns operation, in case of success response will contains message of
	// specified field.
	Response      string `protobuf:"bytes,2,opt,name=response,proto3" json:"response,omitempty"` // Required.
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Operation) Reset() {
	*x = Operation{}
	mi := &file_yandex_cloud_api_operation_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Operation) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Operation) ProtoMessage() {}

func (x *Operation) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_api_operation_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Operation.ProtoReflect.Descriptor instead.
func (*Operation) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_api_operation_proto_rawDescGZIP(), []int{0}
}

func (x *Operation) GetMetadata() string {
	if x != nil {
		return x.Metadata
	}
	return ""
}

func (x *Operation) GetResponse() string {
	if x != nil {
		return x.Response
	}
	return ""
}

var file_yandex_cloud_api_operation_proto_extTypes = []protoimpl.ExtensionInfo{
	{
		ExtendedType:  (*descriptorpb.MethodOptions)(nil),
		ExtensionType: (*Operation)(nil),
		Field:         87334,
		Name:          "yandex.cloud.api.operation",
		Tag:           "bytes,87334,opt,name=operation",
		Filename:      "yandex/cloud/api/operation.proto",
	},
}

// Extension fields to descriptorpb.MethodOptions.
var (
	// optional yandex.cloud.api.Operation operation = 87334;
	E_Operation = &file_yandex_cloud_api_operation_proto_extTypes[0]
)

var File_yandex_cloud_api_operation_proto protoreflect.FileDescriptor

const file_yandex_cloud_api_operation_proto_rawDesc = "" +
	"\n" +
	" yandex/cloud/api/operation.proto\x12\x10yandex.cloud.api\x1a google/protobuf/descriptor.proto\"C\n" +
	"\tOperation\x12\x1a\n" +
	"\bmetadata\x18\x01 \x01(\tR\bmetadata\x12\x1a\n" +
	"\bresponse\x18\x02 \x01(\tR\bresponse:[\n" +
	"\toperation\x12\x1e.google.protobuf.MethodOptions\x18\xa6\xaa\x05 \x01(\v2\x1b.yandex.cloud.api.OperationR\toperationB:Z8github.com/yandex-cloud/go-genproto/yandex/cloud/api;apib\x06proto3"

var (
	file_yandex_cloud_api_operation_proto_rawDescOnce sync.Once
	file_yandex_cloud_api_operation_proto_rawDescData []byte
)

func file_yandex_cloud_api_operation_proto_rawDescGZIP() []byte {
	file_yandex_cloud_api_operation_proto_rawDescOnce.Do(func() {
		file_yandex_cloud_api_operation_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_yandex_cloud_api_operation_proto_rawDesc), len(file_yandex_cloud_api_operation_proto_rawDesc)))
	})
	return file_yandex_cloud_api_operation_proto_rawDescData
}

var file_yandex_cloud_api_operation_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_yandex_cloud_api_operation_proto_goTypes = []any{
	(*Operation)(nil),                  // 0: yandex.cloud.api.Operation
	(*descriptorpb.MethodOptions)(nil), // 1: google.protobuf.MethodOptions
}
var file_yandex_cloud_api_operation_proto_depIdxs = []int32{
	1, // 0: yandex.cloud.api.operation:extendee -> google.protobuf.MethodOptions
	0, // 1: yandex.cloud.api.operation:type_name -> yandex.cloud.api.Operation
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	1, // [1:2] is the sub-list for extension type_name
	0, // [0:1] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_yandex_cloud_api_operation_proto_init() }
func file_yandex_cloud_api_operation_proto_init() {
	if File_yandex_cloud_api_operation_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_yandex_cloud_api_operation_proto_rawDesc), len(file_yandex_cloud_api_operation_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 1,
			NumServices:   0,
		},
		GoTypes:           file_yandex_cloud_api_operation_proto_goTypes,
		DependencyIndexes: file_yandex_cloud_api_operation_proto_depIdxs,
		MessageInfos:      file_yandex_cloud_api_operation_proto_msgTypes,
		ExtensionInfos:    file_yandex_cloud_api_operation_proto_extTypes,
	}.Build()
	File_yandex_cloud_api_operation_proto = out.File
	file_yandex_cloud_api_operation_proto_goTypes = nil
	file_yandex_cloud_api_operation_proto_depIdxs = nil
}
