// Copyright 2022-2025 The sacloud/iaas-api-go Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// generated by 'github.com/sacloud/iaas-api-go/internal/tools/gen-api-meta'; DO NOT EDIT

package fake

const (
	// ResourceArchive is resource key of fake store
	ResourceArchive = "Archive"
	// ResourceAuthStatus is resource key of fake store
	ResourceAuthStatus = "AuthStatus"
	// ResourceAutoBackup is resource key of fake store
	ResourceAutoBackup = "AutoBackup"
	// ResourceAutoScale is resource key of fake store
	ResourceAutoScale = "AutoScale"
	// ResourceBill is resource key of fake store
	ResourceBill = "Bill"
	// ResourceBridge is resource key of fake store
	ResourceBridge = "Bridge"
	// ResourceCDROM is resource key of fake store
	ResourceCDROM = "CDROM"
	// ResourceCertificateAuthority is resource key of fake store
	ResourceCertificateAuthority = "CertificateAuthority"
	// ResourceContainerRegistry is resource key of fake store
	ResourceContainerRegistry = "ContainerRegistry"
	// ResourceCoupon is resource key of fake store
	ResourceCoupon = "Coupon"
	// ResourceDatabase is resource key of fake store
	ResourceDatabase = "Database"
	// ResourceDisk is resource key of fake store
	ResourceDisk = "Disk"
	// ResourceDiskPlan is resource key of fake store
	ResourceDiskPlan = "DiskPlan"
	// ResourceDNS is resource key of fake store
	ResourceDNS = "DNS"
	// ResourceEnhancedDB is resource key of fake store
	ResourceEnhancedDB = "EnhancedDB"
	// ResourceESME is resource key of fake store
	ResourceESME = "ESME"
	// ResourceGSLB is resource key of fake store
	ResourceGSLB = "GSLB"
	// ResourceIcon is resource key of fake store
	ResourceIcon = "Icon"
	// ResourceInterface is resource key of fake store
	ResourceInterface = "Interface"
	// ResourceInternet is resource key of fake store
	ResourceInternet = "Internet"
	// ResourceInternetPlan is resource key of fake store
	ResourceInternetPlan = "InternetPlan"
	// ResourceIPAddress is resource key of fake store
	ResourceIPAddress = "IPAddress"
	// ResourceIPv6Net is resource key of fake store
	ResourceIPv6Net = "IPv6Net"
	// ResourceIPv6Addr is resource key of fake store
	ResourceIPv6Addr = "IPv6Addr"
	// ResourceLicense is resource key of fake store
	ResourceLicense = "License"
	// ResourceLicenseInfo is resource key of fake store
	ResourceLicenseInfo = "LicenseInfo"
	// ResourceLoadBalancer is resource key of fake store
	ResourceLoadBalancer = "LoadBalancer"
	// ResourceLocalRouter is resource key of fake store
	ResourceLocalRouter = "LocalRouter"
	// ResourceMobileGateway is resource key of fake store
	ResourceMobileGateway = "MobileGateway"
	// ResourceNFS is resource key of fake store
	ResourceNFS = "NFS"
	// ResourceNote is resource key of fake store
	ResourceNote = "Note"
	// ResourcePacketFilter is resource key of fake store
	ResourcePacketFilter = "PacketFilter"
	// ResourcePrivateHost is resource key of fake store
	ResourcePrivateHost = "PrivateHost"
	// ResourcePrivateHostPlan is resource key of fake store
	ResourcePrivateHostPlan = "PrivateHostPlan"
	// ResourceProxyLB is resource key of fake store
	ResourceProxyLB = "ProxyLB"
	// ResourceRegion is resource key of fake store
	ResourceRegion = "Region"
	// ResourceServer is resource key of fake store
	ResourceServer = "Server"
	// ResourceServerPlan is resource key of fake store
	ResourceServerPlan = "ServerPlan"
	// ResourceServiceClass is resource key of fake store
	ResourceServiceClass = "ServiceClass"
	// ResourceSIM is resource key of fake store
	ResourceSIM = "SIM"
	// ResourceSimpleMonitor is resource key of fake store
	ResourceSimpleMonitor = "SimpleMonitor"
	// ResourceSimpleNotificationDestination is resource key of fake store
	ResourceSimpleNotificationDestination = "SimpleNotificationDestination"
	// ResourceSimpleNotificationGroup is resource key of fake store
	ResourceSimpleNotificationGroup = "SimpleNotificationGroup"
	// ResourceSSHKey is resource key of fake store
	ResourceSSHKey = "SSHKey"
	// ResourceSubnet is resource key of fake store
	ResourceSubnet = "Subnet"
	// ResourceSwitch is resource key of fake store
	ResourceSwitch = "Switch"
	// ResourceVPCRouter is resource key of fake store
	ResourceVPCRouter = "VPCRouter"
	// ResourceZone is resource key of fake store
	ResourceZone = "Zone"
)
