// Copyright 2022-2025 The sacloud/iaas-api-go Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package types

// EPlanGeneration サーバプラン世代
type EPlanGeneration int

var (
	// PlanGenerationValues 有効なサーバプラン世代の値
	PlanGenerationValues = []int{
		int(PlanGenerations.Default),
		int(PlanGenerations.G100),
		int(PlanGenerations.G200),
	}

	// PlanGenerations サーバプラン世代
	PlanGenerations = struct {
		// Default デフォルト(自動選択)
		Default EPlanGeneration
		// G100 第1世代
		G100 EPlanGeneration
		// G200 第2世代
		G200 EPlanGeneration
	}{
		Default: EPlanGeneration(0),
		G100:    EPlanGeneration(100),
		G200:    EPlanGeneration(200),
	}
)
