// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: yandex/cloud/iam/v1/service_account_service.proto

package iam

import (
	_ "github.com/yandex-cloud/go-genproto/yandex/cloud"
	access "github.com/yandex-cloud/go-genproto/yandex/cloud/access"
	_ "github.com/yandex-cloud/go-genproto/yandex/cloud/api"
	operation "github.com/yandex-cloud/go-genproto/yandex/cloud/operation"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	fieldmaskpb "google.golang.org/protobuf/types/known/fieldmaskpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type GetServiceAccountRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the ServiceAccount resource to return.
	// To get the service account ID, use a [ServiceAccountService.List] request.
	ServiceAccountId string `protobuf:"bytes,1,opt,name=service_account_id,json=serviceAccountId,proto3" json:"service_account_id,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *GetServiceAccountRequest) Reset() {
	*x = GetServiceAccountRequest{}
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetServiceAccountRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetServiceAccountRequest) ProtoMessage() {}

func (x *GetServiceAccountRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetServiceAccountRequest.ProtoReflect.Descriptor instead.
func (*GetServiceAccountRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_account_service_proto_rawDescGZIP(), []int{0}
}

func (x *GetServiceAccountRequest) GetServiceAccountId() string {
	if x != nil {
		return x.ServiceAccountId
	}
	return ""
}

type ListServiceAccountsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the folder to list service accounts in.
	// To get the folder ID, use a [yandex.cloud.resourcemanager.v1.FolderService.List] request.
	FolderId string `protobuf:"bytes,1,opt,name=folder_id,json=folderId,proto3" json:"folder_id,omitempty"`
	// The maximum number of results per page to return. If the number of available
	// results is larger than [page_size],
	// the service returns a [ListServiceAccountsResponse.next_page_token]
	// that can be used to get the next page of results in subsequent list requests.
	// Default value: 100
	PageSize int64 `protobuf:"varint,2,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Page token. To get the next page of results, set [page_token]
	// to the [ListServiceAccountsResponse.next_page_token]
	// returned by a previous list request.
	PageToken string `protobuf:"bytes,3,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	// A filter expression that filters resources listed in the response.
	// The expression must specify:
	// 1. The field name. Currently you can use filtering only on the [ServiceAccount.name] field.
	// 2. An `=` operator.
	// 3. The value in double quotes (`"`). Must be 3-63 characters long and match the regular expression `[a-z][-a-z0-9]{1,61}[a-z0-9]`.
	Filter        string `protobuf:"bytes,4,opt,name=filter,proto3" json:"filter,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListServiceAccountsRequest) Reset() {
	*x = ListServiceAccountsRequest{}
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListServiceAccountsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListServiceAccountsRequest) ProtoMessage() {}

func (x *ListServiceAccountsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListServiceAccountsRequest.ProtoReflect.Descriptor instead.
func (*ListServiceAccountsRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_account_service_proto_rawDescGZIP(), []int{1}
}

func (x *ListServiceAccountsRequest) GetFolderId() string {
	if x != nil {
		return x.FolderId
	}
	return ""
}

func (x *ListServiceAccountsRequest) GetPageSize() int64 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListServiceAccountsRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

func (x *ListServiceAccountsRequest) GetFilter() string {
	if x != nil {
		return x.Filter
	}
	return ""
}

type ListServiceAccountsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// List of ServiceAccount resources.
	ServiceAccounts []*ServiceAccount `protobuf:"bytes,1,rep,name=service_accounts,json=serviceAccounts,proto3" json:"service_accounts,omitempty"`
	// This token allows you to get the next page of results for list requests. If the number of results
	// is larger than [ListServiceAccountsRequest.page_size], use
	// the [next_page_token] as the value
	// for the [ListServiceAccountsRequest.page_token] query parameter
	// in the next list request. Each subsequent list request will have its own
	// [next_page_token] to continue paging through the results.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListServiceAccountsResponse) Reset() {
	*x = ListServiceAccountsResponse{}
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListServiceAccountsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListServiceAccountsResponse) ProtoMessage() {}

func (x *ListServiceAccountsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListServiceAccountsResponse.ProtoReflect.Descriptor instead.
func (*ListServiceAccountsResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_account_service_proto_rawDescGZIP(), []int{2}
}

func (x *ListServiceAccountsResponse) GetServiceAccounts() []*ServiceAccount {
	if x != nil {
		return x.ServiceAccounts
	}
	return nil
}

func (x *ListServiceAccountsResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

type CreateServiceAccountRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the folder to create a service account in.
	// To get the folder ID, use a [yandex.cloud.resourcemanager.v1.FolderService.List] request.
	FolderId string `protobuf:"bytes,1,opt,name=folder_id,json=folderId,proto3" json:"folder_id,omitempty"`
	// Name of the service account.
	// The name must be unique within the cloud.
	Name string `protobuf:"bytes,2,opt,name=name,proto3" json:"name,omitempty"`
	// Description of the service account.
	Description string `protobuf:"bytes,3,opt,name=description,proto3" json:"description,omitempty"`
	// Resource labels as “ key:value “ pairs.
	Labels        map[string]string `protobuf:"bytes,4,rep,name=labels,proto3" json:"labels,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateServiceAccountRequest) Reset() {
	*x = CreateServiceAccountRequest{}
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateServiceAccountRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateServiceAccountRequest) ProtoMessage() {}

func (x *CreateServiceAccountRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateServiceAccountRequest.ProtoReflect.Descriptor instead.
func (*CreateServiceAccountRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_account_service_proto_rawDescGZIP(), []int{3}
}

func (x *CreateServiceAccountRequest) GetFolderId() string {
	if x != nil {
		return x.FolderId
	}
	return ""
}

func (x *CreateServiceAccountRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *CreateServiceAccountRequest) GetDescription() string {
	if x != nil {
		return x.Description
	}
	return ""
}

func (x *CreateServiceAccountRequest) GetLabels() map[string]string {
	if x != nil {
		return x.Labels
	}
	return nil
}

type CreateServiceAccountMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the service account that is being created.
	ServiceAccountId string `protobuf:"bytes,1,opt,name=service_account_id,json=serviceAccountId,proto3" json:"service_account_id,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *CreateServiceAccountMetadata) Reset() {
	*x = CreateServiceAccountMetadata{}
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateServiceAccountMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateServiceAccountMetadata) ProtoMessage() {}

func (x *CreateServiceAccountMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateServiceAccountMetadata.ProtoReflect.Descriptor instead.
func (*CreateServiceAccountMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_account_service_proto_rawDescGZIP(), []int{4}
}

func (x *CreateServiceAccountMetadata) GetServiceAccountId() string {
	if x != nil {
		return x.ServiceAccountId
	}
	return ""
}

type UpdateServiceAccountRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the ServiceAccount resource to update.
	// To get the service account ID, use a [ServiceAccountService.List] request.
	ServiceAccountId string `protobuf:"bytes,1,opt,name=service_account_id,json=serviceAccountId,proto3" json:"service_account_id,omitempty"`
	// Field mask that specifies which fields of the ServiceAccount resource are going to be updated.
	UpdateMask *fieldmaskpb.FieldMask `protobuf:"bytes,2,opt,name=update_mask,json=updateMask,proto3" json:"update_mask,omitempty"`
	// Name of the service account.
	// The name must be unique within the cloud.
	Name string `protobuf:"bytes,3,opt,name=name,proto3" json:"name,omitempty"`
	// Description of the service account.
	Description string `protobuf:"bytes,4,opt,name=description,proto3" json:"description,omitempty"`
	// Resource labels as “ key:value “ pairs.
	Labels        map[string]string `protobuf:"bytes,5,rep,name=labels,proto3" json:"labels,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateServiceAccountRequest) Reset() {
	*x = UpdateServiceAccountRequest{}
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateServiceAccountRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateServiceAccountRequest) ProtoMessage() {}

func (x *UpdateServiceAccountRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateServiceAccountRequest.ProtoReflect.Descriptor instead.
func (*UpdateServiceAccountRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_account_service_proto_rawDescGZIP(), []int{5}
}

func (x *UpdateServiceAccountRequest) GetServiceAccountId() string {
	if x != nil {
		return x.ServiceAccountId
	}
	return ""
}

func (x *UpdateServiceAccountRequest) GetUpdateMask() *fieldmaskpb.FieldMask {
	if x != nil {
		return x.UpdateMask
	}
	return nil
}

func (x *UpdateServiceAccountRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *UpdateServiceAccountRequest) GetDescription() string {
	if x != nil {
		return x.Description
	}
	return ""
}

func (x *UpdateServiceAccountRequest) GetLabels() map[string]string {
	if x != nil {
		return x.Labels
	}
	return nil
}

type UpdateServiceAccountMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the ServiceAccount resource that is being updated.
	ServiceAccountId string `protobuf:"bytes,1,opt,name=service_account_id,json=serviceAccountId,proto3" json:"service_account_id,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *UpdateServiceAccountMetadata) Reset() {
	*x = UpdateServiceAccountMetadata{}
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateServiceAccountMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateServiceAccountMetadata) ProtoMessage() {}

func (x *UpdateServiceAccountMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateServiceAccountMetadata.ProtoReflect.Descriptor instead.
func (*UpdateServiceAccountMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_account_service_proto_rawDescGZIP(), []int{6}
}

func (x *UpdateServiceAccountMetadata) GetServiceAccountId() string {
	if x != nil {
		return x.ServiceAccountId
	}
	return ""
}

type DeleteServiceAccountRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the service account to delete.
	// To get the service account ID, use a [ServiceAccountService.List] request.
	ServiceAccountId string `protobuf:"bytes,1,opt,name=service_account_id,json=serviceAccountId,proto3" json:"service_account_id,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *DeleteServiceAccountRequest) Reset() {
	*x = DeleteServiceAccountRequest{}
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DeleteServiceAccountRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DeleteServiceAccountRequest) ProtoMessage() {}

func (x *DeleteServiceAccountRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DeleteServiceAccountRequest.ProtoReflect.Descriptor instead.
func (*DeleteServiceAccountRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_account_service_proto_rawDescGZIP(), []int{7}
}

func (x *DeleteServiceAccountRequest) GetServiceAccountId() string {
	if x != nil {
		return x.ServiceAccountId
	}
	return ""
}

type DeleteServiceAccountMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the service account that is being deleted.
	ServiceAccountId string `protobuf:"bytes,1,opt,name=service_account_id,json=serviceAccountId,proto3" json:"service_account_id,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *DeleteServiceAccountMetadata) Reset() {
	*x = DeleteServiceAccountMetadata{}
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DeleteServiceAccountMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DeleteServiceAccountMetadata) ProtoMessage() {}

func (x *DeleteServiceAccountMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DeleteServiceAccountMetadata.ProtoReflect.Descriptor instead.
func (*DeleteServiceAccountMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_account_service_proto_rawDescGZIP(), []int{8}
}

func (x *DeleteServiceAccountMetadata) GetServiceAccountId() string {
	if x != nil {
		return x.ServiceAccountId
	}
	return ""
}

type ListServiceAccountOperationsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the ServiceAccount resource to list operations for.
	ServiceAccountId string `protobuf:"bytes,1,opt,name=service_account_id,json=serviceAccountId,proto3" json:"service_account_id,omitempty"`
	// The maximum number of results per page to return. If the number of available
	// results is larger than [page_size], the service returns a [ListServiceAccountOperationsResponse.next_page_token]
	// that can be used to get the next page of results in subsequent list requests.
	// Default value: 100.
	PageSize int64 `protobuf:"varint,2,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Page token. To get the next page of results, set [page_token]
	// to the [ListServiceAccountOperationsResponse.next_page_token]
	// returned by a previous list request.
	PageToken     string `protobuf:"bytes,3,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListServiceAccountOperationsRequest) Reset() {
	*x = ListServiceAccountOperationsRequest{}
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListServiceAccountOperationsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListServiceAccountOperationsRequest) ProtoMessage() {}

func (x *ListServiceAccountOperationsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListServiceAccountOperationsRequest.ProtoReflect.Descriptor instead.
func (*ListServiceAccountOperationsRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_account_service_proto_rawDescGZIP(), []int{9}
}

func (x *ListServiceAccountOperationsRequest) GetServiceAccountId() string {
	if x != nil {
		return x.ServiceAccountId
	}
	return ""
}

func (x *ListServiceAccountOperationsRequest) GetPageSize() int64 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListServiceAccountOperationsRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

type ListServiceAccountOperationsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// List of operations for the specified service account.
	Operations []*operation.Operation `protobuf:"bytes,1,rep,name=operations,proto3" json:"operations,omitempty"`
	// This token allows you to get the next page of results for list requests. If the number of results
	// is larger than [ListServiceAccountOperationsRequest.page_size], use the [next_page_token] as the value
	// for the [ListServiceAccountOperationsRequest.page_token] query parameter in the next list request.
	// Each subsequent list request will have its own [next_page_token] to continue paging through the results.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListServiceAccountOperationsResponse) Reset() {
	*x = ListServiceAccountOperationsResponse{}
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListServiceAccountOperationsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListServiceAccountOperationsResponse) ProtoMessage() {}

func (x *ListServiceAccountOperationsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListServiceAccountOperationsResponse.ProtoReflect.Descriptor instead.
func (*ListServiceAccountOperationsResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_account_service_proto_rawDescGZIP(), []int{10}
}

func (x *ListServiceAccountOperationsResponse) GetOperations() []*operation.Operation {
	if x != nil {
		return x.Operations
	}
	return nil
}

func (x *ListServiceAccountOperationsResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

var File_yandex_cloud_iam_v1_service_account_service_proto protoreflect.FileDescriptor

const file_yandex_cloud_iam_v1_service_account_service_proto_rawDesc = "" +
	"\n" +
	"1yandex/cloud/iam/v1/service_account_service.proto\x12\x13yandex.cloud.iam.v1\x1a\x1cgoogle/api/annotations.proto\x1a google/protobuf/field_mask.proto\x1a yandex/cloud/access/access.proto\x1a yandex/cloud/api/operation.proto\x1a)yandex/cloud/iam/v1/service_account.proto\x1a&yandex/cloud/operation/operation.proto\x1a\x1dyandex/cloud/validation.proto\"V\n" +
	"\x18GetServiceAccountRequest\x12:\n" +
	"\x12service_account_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x10serviceAccountId\"\xbf\x01\n" +
	"\x1aListServiceAccountsRequest\x12)\n" +
	"\tfolder_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\bfolderId\x12'\n" +
	"\tpage_size\x18\x02 \x01(\x03B\n" +
	"\xfa\xc71\x060-1000R\bpageSize\x12)\n" +
	"\n" +
	"page_token\x18\x03 \x01(\tB\n" +
	"\x8a\xc81\x06<=2000R\tpageToken\x12\"\n" +
	"\x06filter\x18\x04 \x01(\tB\n" +
	"\x8a\xc81\x06<=1000R\x06filter\"\x95\x01\n" +
	"\x1bListServiceAccountsResponse\x12N\n" +
	"\x10service_accounts\x18\x01 \x03(\v2#.yandex.cloud.iam.v1.ServiceAccountR\x0fserviceAccounts\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\x81\x03\n" +
	"\x1bCreateServiceAccountRequest\x12)\n" +
	"\tfolder_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\bfolderId\x12;\n" +
	"\x04name\x18\x02 \x01(\tB'\xe8\xc71\x01\xf2\xc71\x1f[a-z]([-a-z0-9]{0,61}[a-z0-9])?R\x04name\x12+\n" +
	"\vdescription\x18\x03 \x01(\tB\t\x8a\xc81\x05<=256R\vdescription\x12\x91\x01\n" +
	"\x06labels\x18\x04 \x03(\v2<.yandex.cloud.iam.v1.CreateServiceAccountRequest.LabelsEntryB;\xf2\xc71\v[-_0-9a-z]*\x82\xc81\x04<=64\x8a\xc81\x04<=63\xb2\xc81\x18\x12\x10[a-z][-_0-9a-z]*\x1a\x041-63R\x06labels\x1a9\n" +
	"\vLabelsEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\tR\x05value:\x028\x01\"L\n" +
	"\x1cCreateServiceAccountMetadata\x12,\n" +
	"\x12service_account_id\x18\x01 \x01(\tR\x10serviceAccountId\"\xcf\x03\n" +
	"\x1bUpdateServiceAccountRequest\x12:\n" +
	"\x12service_account_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x10serviceAccountId\x12;\n" +
	"\vupdate_mask\x18\x02 \x01(\v2\x1a.google.protobuf.FieldMaskR\n" +
	"updateMask\x12;\n" +
	"\x04name\x18\x03 \x01(\tB'\xe8\xc71\x01\xf2\xc71\x1f[a-z]([-a-z0-9]{0,61}[a-z0-9])?R\x04name\x12+\n" +
	"\vdescription\x18\x04 \x01(\tB\t\x8a\xc81\x05<=256R\vdescription\x12\x91\x01\n" +
	"\x06labels\x18\x05 \x03(\v2<.yandex.cloud.iam.v1.UpdateServiceAccountRequest.LabelsEntryB;\xf2\xc71\v[-_0-9a-z]*\x82\xc81\x04<=64\x8a\xc81\x04<=63\xb2\xc81\x18\x12\x10[a-z][-_0-9a-z]*\x1a\x041-63R\x06labels\x1a9\n" +
	"\vLabelsEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\tR\x05value:\x028\x01\"L\n" +
	"\x1cUpdateServiceAccountMetadata\x12,\n" +
	"\x12service_account_id\x18\x01 \x01(\tR\x10serviceAccountId\"Y\n" +
	"\x1bDeleteServiceAccountRequest\x12:\n" +
	"\x12service_account_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x10serviceAccountId\"L\n" +
	"\x1cDeleteServiceAccountMetadata\x12,\n" +
	"\x12service_account_id\x18\x01 \x01(\tR\x10serviceAccountId\"\xb5\x01\n" +
	"#ListServiceAccountOperationsRequest\x12:\n" +
	"\x12service_account_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x10serviceAccountId\x12'\n" +
	"\tpage_size\x18\x02 \x01(\x03B\n" +
	"\xfa\xc71\x060-1000R\bpageSize\x12)\n" +
	"\n" +
	"page_token\x18\x03 \x01(\tB\n" +
	"\x8a\xc81\x06<=2000R\tpageToken\"\x91\x01\n" +
	"$ListServiceAccountOperationsResponse\x12A\n" +
	"\n" +
	"operations\x18\x01 \x03(\v2!.yandex.cloud.operation.OperationR\n" +
	"operations\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\x87\x0e\n" +
	"\x15ServiceAccountService\x12\x8f\x01\n" +
	"\x03Get\x12-.yandex.cloud.iam.v1.GetServiceAccountRequest\x1a#.yandex.cloud.iam.v1.ServiceAccount\"4\x82\xd3\xe4\x93\x02.\x12,/iam/v1/serviceAccounts/{service_account_id}\x12\x8a\x01\n" +
	"\x04List\x12/.yandex.cloud.iam.v1.ListServiceAccountsRequest\x1a0.yandex.cloud.iam.v1.ListServiceAccountsResponse\"\x1f\x82\xd3\xe4\x93\x02\x19\x12\x17/iam/v1/serviceAccounts\x12\xb3\x01\n" +
	"\x06Create\x120.yandex.cloud.iam.v1.CreateServiceAccountRequest\x1a!.yandex.cloud.operation.Operation\"T\xb2\xd2*.\n" +
	"\x1cCreateServiceAccountMetadata\x12\x0eServiceAccount\x82\xd3\xe4\x93\x02\x1c:\x01*\"\x17/iam/v1/serviceAccounts\x12\xc8\x01\n" +
	"\x06Update\x120.yandex.cloud.iam.v1.UpdateServiceAccountRequest\x1a!.yandex.cloud.operation.Operation\"i\xb2\xd2*.\n" +
	"\x1cUpdateServiceAccountMetadata\x12\x0eServiceAccount\x82\xd3\xe4\x93\x021:\x01*2,/iam/v1/serviceAccounts/{service_account_id}\x12\xcc\x01\n" +
	"\x06Delete\x120.yandex.cloud.iam.v1.DeleteServiceAccountRequest\x1a!.yandex.cloud.operation.Operation\"m\xb2\xd2*5\n" +
	"\x1cDeleteServiceAccountMetadata\x12\x15google.protobuf.Empty\x82\xd3\xe4\x93\x02.*,/iam/v1/serviceAccounts/{service_account_id}\x12\xb7\x01\n" +
	"\x12ListAccessBindings\x12..yandex.cloud.access.ListAccessBindingsRequest\x1a/.yandex.cloud.access.ListAccessBindingsResponse\"@\x82\xd3\xe4\x93\x02:\x128/iam/v1/serviceAccounts/{resource_id}:listAccessBindings\x12\xf6\x01\n" +
	"\x11SetAccessBindings\x12-.yandex.cloud.access.SetAccessBindingsRequest\x1a!.yandex.cloud.operation.Operation\"\x8e\x01\xb2\xd2*H\n" +
	" access.SetAccessBindingsMetadata\x12$access.AccessBindingsOperationResult\x82\xd3\xe4\x93\x02<:\x01*\"7/iam/v1/serviceAccounts/{resource_id}:setAccessBindings\x12\x82\x02\n" +
	"\x14UpdateAccessBindings\x120.yandex.cloud.access.UpdateAccessBindingsRequest\x1a!.yandex.cloud.operation.Operation\"\x94\x01\xb2\xd2*K\n" +
	"#access.UpdateAccessBindingsMetadata\x12$access.AccessBindingsOperationResult\x82\xd3\xe4\x93\x02?:\x01*\":/iam/v1/serviceAccounts/{resource_id}:updateAccessBindings\x12\xc6\x01\n" +
	"\x0eListOperations\x128.yandex.cloud.iam.v1.ListServiceAccountOperationsRequest\x1a9.yandex.cloud.iam.v1.ListServiceAccountOperationsResponse\"?\x82\xd3\xe4\x93\x029\x127/iam/v1/serviceAccounts/{service_account_id}/operationsBV\n" +
	"\x17yandex.cloud.api.iam.v1Z;github.com/yandex-cloud/go-genproto/yandex/cloud/iam/v1;iamb\x06proto3"

var (
	file_yandex_cloud_iam_v1_service_account_service_proto_rawDescOnce sync.Once
	file_yandex_cloud_iam_v1_service_account_service_proto_rawDescData []byte
)

func file_yandex_cloud_iam_v1_service_account_service_proto_rawDescGZIP() []byte {
	file_yandex_cloud_iam_v1_service_account_service_proto_rawDescOnce.Do(func() {
		file_yandex_cloud_iam_v1_service_account_service_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_service_account_service_proto_rawDesc), len(file_yandex_cloud_iam_v1_service_account_service_proto_rawDesc)))
	})
	return file_yandex_cloud_iam_v1_service_account_service_proto_rawDescData
}

var file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes = make([]protoimpl.MessageInfo, 13)
var file_yandex_cloud_iam_v1_service_account_service_proto_goTypes = []any{
	(*GetServiceAccountRequest)(nil),             // 0: yandex.cloud.iam.v1.GetServiceAccountRequest
	(*ListServiceAccountsRequest)(nil),           // 1: yandex.cloud.iam.v1.ListServiceAccountsRequest
	(*ListServiceAccountsResponse)(nil),          // 2: yandex.cloud.iam.v1.ListServiceAccountsResponse
	(*CreateServiceAccountRequest)(nil),          // 3: yandex.cloud.iam.v1.CreateServiceAccountRequest
	(*CreateServiceAccountMetadata)(nil),         // 4: yandex.cloud.iam.v1.CreateServiceAccountMetadata
	(*UpdateServiceAccountRequest)(nil),          // 5: yandex.cloud.iam.v1.UpdateServiceAccountRequest
	(*UpdateServiceAccountMetadata)(nil),         // 6: yandex.cloud.iam.v1.UpdateServiceAccountMetadata
	(*DeleteServiceAccountRequest)(nil),          // 7: yandex.cloud.iam.v1.DeleteServiceAccountRequest
	(*DeleteServiceAccountMetadata)(nil),         // 8: yandex.cloud.iam.v1.DeleteServiceAccountMetadata
	(*ListServiceAccountOperationsRequest)(nil),  // 9: yandex.cloud.iam.v1.ListServiceAccountOperationsRequest
	(*ListServiceAccountOperationsResponse)(nil), // 10: yandex.cloud.iam.v1.ListServiceAccountOperationsResponse
	nil,                                      // 11: yandex.cloud.iam.v1.CreateServiceAccountRequest.LabelsEntry
	nil,                                      // 12: yandex.cloud.iam.v1.UpdateServiceAccountRequest.LabelsEntry
	(*ServiceAccount)(nil),                   // 13: yandex.cloud.iam.v1.ServiceAccount
	(*fieldmaskpb.FieldMask)(nil),            // 14: google.protobuf.FieldMask
	(*operation.Operation)(nil),              // 15: yandex.cloud.operation.Operation
	(*access.ListAccessBindingsRequest)(nil), // 16: yandex.cloud.access.ListAccessBindingsRequest
	(*access.SetAccessBindingsRequest)(nil),  // 17: yandex.cloud.access.SetAccessBindingsRequest
	(*access.UpdateAccessBindingsRequest)(nil), // 18: yandex.cloud.access.UpdateAccessBindingsRequest
	(*access.ListAccessBindingsResponse)(nil),  // 19: yandex.cloud.access.ListAccessBindingsResponse
}
var file_yandex_cloud_iam_v1_service_account_service_proto_depIdxs = []int32{
	13, // 0: yandex.cloud.iam.v1.ListServiceAccountsResponse.service_accounts:type_name -> yandex.cloud.iam.v1.ServiceAccount
	11, // 1: yandex.cloud.iam.v1.CreateServiceAccountRequest.labels:type_name -> yandex.cloud.iam.v1.CreateServiceAccountRequest.LabelsEntry
	14, // 2: yandex.cloud.iam.v1.UpdateServiceAccountRequest.update_mask:type_name -> google.protobuf.FieldMask
	12, // 3: yandex.cloud.iam.v1.UpdateServiceAccountRequest.labels:type_name -> yandex.cloud.iam.v1.UpdateServiceAccountRequest.LabelsEntry
	15, // 4: yandex.cloud.iam.v1.ListServiceAccountOperationsResponse.operations:type_name -> yandex.cloud.operation.Operation
	0,  // 5: yandex.cloud.iam.v1.ServiceAccountService.Get:input_type -> yandex.cloud.iam.v1.GetServiceAccountRequest
	1,  // 6: yandex.cloud.iam.v1.ServiceAccountService.List:input_type -> yandex.cloud.iam.v1.ListServiceAccountsRequest
	3,  // 7: yandex.cloud.iam.v1.ServiceAccountService.Create:input_type -> yandex.cloud.iam.v1.CreateServiceAccountRequest
	5,  // 8: yandex.cloud.iam.v1.ServiceAccountService.Update:input_type -> yandex.cloud.iam.v1.UpdateServiceAccountRequest
	7,  // 9: yandex.cloud.iam.v1.ServiceAccountService.Delete:input_type -> yandex.cloud.iam.v1.DeleteServiceAccountRequest
	16, // 10: yandex.cloud.iam.v1.ServiceAccountService.ListAccessBindings:input_type -> yandex.cloud.access.ListAccessBindingsRequest
	17, // 11: yandex.cloud.iam.v1.ServiceAccountService.SetAccessBindings:input_type -> yandex.cloud.access.SetAccessBindingsRequest
	18, // 12: yandex.cloud.iam.v1.ServiceAccountService.UpdateAccessBindings:input_type -> yandex.cloud.access.UpdateAccessBindingsRequest
	9,  // 13: yandex.cloud.iam.v1.ServiceAccountService.ListOperations:input_type -> yandex.cloud.iam.v1.ListServiceAccountOperationsRequest
	13, // 14: yandex.cloud.iam.v1.ServiceAccountService.Get:output_type -> yandex.cloud.iam.v1.ServiceAccount
	2,  // 15: yandex.cloud.iam.v1.ServiceAccountService.List:output_type -> yandex.cloud.iam.v1.ListServiceAccountsResponse
	15, // 16: yandex.cloud.iam.v1.ServiceAccountService.Create:output_type -> yandex.cloud.operation.Operation
	15, // 17: yandex.cloud.iam.v1.ServiceAccountService.Update:output_type -> yandex.cloud.operation.Operation
	15, // 18: yandex.cloud.iam.v1.ServiceAccountService.Delete:output_type -> yandex.cloud.operation.Operation
	19, // 19: yandex.cloud.iam.v1.ServiceAccountService.ListAccessBindings:output_type -> yandex.cloud.access.ListAccessBindingsResponse
	15, // 20: yandex.cloud.iam.v1.ServiceAccountService.SetAccessBindings:output_type -> yandex.cloud.operation.Operation
	15, // 21: yandex.cloud.iam.v1.ServiceAccountService.UpdateAccessBindings:output_type -> yandex.cloud.operation.Operation
	10, // 22: yandex.cloud.iam.v1.ServiceAccountService.ListOperations:output_type -> yandex.cloud.iam.v1.ListServiceAccountOperationsResponse
	14, // [14:23] is the sub-list for method output_type
	5,  // [5:14] is the sub-list for method input_type
	5,  // [5:5] is the sub-list for extension type_name
	5,  // [5:5] is the sub-list for extension extendee
	0,  // [0:5] is the sub-list for field type_name
}

func init() { file_yandex_cloud_iam_v1_service_account_service_proto_init() }
func file_yandex_cloud_iam_v1_service_account_service_proto_init() {
	if File_yandex_cloud_iam_v1_service_account_service_proto != nil {
		return
	}
	file_yandex_cloud_iam_v1_service_account_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_service_account_service_proto_rawDesc), len(file_yandex_cloud_iam_v1_service_account_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   13,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_yandex_cloud_iam_v1_service_account_service_proto_goTypes,
		DependencyIndexes: file_yandex_cloud_iam_v1_service_account_service_proto_depIdxs,
		MessageInfos:      file_yandex_cloud_iam_v1_service_account_service_proto_msgTypes,
	}.Build()
	File_yandex_cloud_iam_v1_service_account_service_proto = out.File
	file_yandex_cloud_iam_v1_service_account_service_proto_goTypes = nil
	file_yandex_cloud_iam_v1_service_account_service_proto_depIdxs = nil
}
