// Code generated by protoc-gen-goext. DO NOT EDIT.

package iam

import (
	access "github.com/yandex-cloud/go-genproto/yandex/cloud/access"
)

func (m *ListAccessPolicyTemplatesRequest) SetPageSize(v int64) {
	m.PageSize = v
}

func (m *ListAccessPolicyTemplatesRequest) SetPageToken(v string) {
	m.PageToken = v
}

func (m *ListAccessPolicyTemplatesResponse) SetAccessPolicyTemplates(v []*access.AccessPolicy) {
	m.AccessPolicyTemplates = v
}

func (m *ListAccessPolicyTemplatesResponse) SetNextPageToken(v string) {
	m.NextPageToken = v
}
