// Code generated by protoc-gen-goext. DO NOT EDIT.

package operation

func (m *GetOperationRequest) SetOperationId(v string) {
	m.OperationId = v
}

func (m *CancelOperationRequest) SetOperationId(v string) {
	m.OperationId = v
}
