// Copyright 2018 JDCLOUD.COM
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// NOTE: This class is auto generated by the jdcloud code generator program.

package models

type SetRecords struct {

	/* 解析记录的值  */
	Records []string `json:"records"`

	/* 解析记录所在的解析线路，请调用describeViewTree接口获取。如果不传是默认线路。 (Optional) */
	Views []string `json:"views"`

	/* 解析记录的类型，支持A, AAAA, CNAME类型  */
	Type string `json:"type"`
}
