// Code generated by protoc-gen-goext. DO NOT EDIT.

package iam

import (
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
)

func (m *ServiceAccount) SetId(v string) {
	m.Id = v
}

func (m *ServiceAccount) SetFolderId(v string) {
	m.FolderId = v
}

func (m *ServiceAccount) SetCreatedAt(v *timestamppb.Timestamp) {
	m.CreatedAt = v
}

func (m *ServiceAccount) SetName(v string) {
	m.Name = v
}

func (m *ServiceAccount) SetDescription(v string) {
	m.Description = v
}

func (m *ServiceAccount) SetLabels(v map[string]string) {
	m.Labels = v
}

func (m *ServiceAccount) SetLastAuthenticatedAt(v *timestamppb.Timestamp) {
	m.LastAuthenticatedAt = v
}
