// Copyright 2018 JDCLOUD.COM
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// NOTE: This class is auto generated by the jdcloud code generator program.

package models

type Monitor struct {

	/* 连续几次触发报警 (Optional) */
	AlarmLimit int `json:"alarmLimit"`

	/* 现在是否可以恢复 (Optional) */
	CanRecover bool `json:"canRecover"`

	/* 现在是否可以切换 (Optional) */
	CanSwitch bool `json:"canSwitch"`

	/* 机房探测点的集合 (Optional) */
	Clusters string `json:"clusters"`

	/* 主域名 (Optional) */
	DomainName string `json:"domainName"`

	/* 主机状态，0正常，1异常 (Optional) */
	HostStatus int `json:"hostStatus"`

	/* 监控对象 (Optional) */
	HostValue string `json:"hostValue"`

	/* 监控项ID (Optional) */
	Id int `json:"id"`

	/* 备用地址1 (Optional) */
	IpBackup01 string `json:"ipBackup01"`

	/* 备用地址1的状态，0正常，1异常 (Optional) */
	IpBackup01Status int `json:"ipBackup01Status"`

	/* 备用地址1的类型，1为ip 2为域名 (Optional) */
	IpBackup01Type int `json:"ipBackup01Type"`

	/* 备用地址2 (Optional) */
	IpBackup02 string `json:"ipBackup02"`

	/* 备用地址2的状态，0正常，1异常 (Optional) */
	IpBackup02Status int `json:"ipBackup02Status"`

	/* 备用地址1的类型，1为ip 2为域名 (Optional) */
	IpBackup02Type int `json:"ipBackup02Type"`

	/* 手动切换的地址 (Optional) */
	ManualBackup string `json:"manualBackup"`

	/* 手动切换的地址的状态，0正常，1异常 (Optional) */
	ManualBackupStatus int `json:"manualBackupStatus"`

	/* 手动切换的地址的类型，1为ip 2为域名 (Optional) */
	ManualBackupType int `json:"manualBackupType"`

	/* 监控状况 开启监控 2，暂停监控 4 (Optional) */
	MonitorEnable int `json:"monitorEnable"`

	/* 监控频率，单位秒 (Optional) */
	MonitorFreq int `json:"monitorFreq"`

	/* 监控端口 (Optional) */
	MonitorPort int `json:"monitorPort"`

	/* 不做任何修改0，强制暂停解析记录1，自动切换到备用地址2 (Optional) */
	MonitorRule int `json:"monitorRule"`

	/* 监控路径 (Optional) */
	MonitorUri string `json:"monitorUri"`

	/* 邮箱地址 (Optional) */
	NotifyEmail string `json:"notifyEmail"`

	/* 不发送邮件0， 发送邮件1 (Optional) */
	NotifyEmailEnable int `json:"notifyEmailEnable"`

	/* 不发送通知栏 0， 发送通知栏 1 (Optional) */
	NotifyMsgBarEnable int `json:"notifyMsgBarEnable"`

	/* 手机号码 (Optional) */
	NotifySms string `json:"notifySms"`

	/* 不发送短信 0， 发送短信 1 (Optional) */
	NotifySmsEnable int `json:"notifySmsEnable"`

	/* http协议:0，https协议:1 (Optional) */
	Protocol int `json:"protocol"`

	/* 自动恢复:0 手动恢复:1 (Optional) */
	StopRecoverRule int `json:"stopRecoverRule"`

	/* 子域名 (Optional) */
	SubDomainName string `json:"subDomainName"`

	/* 自动恢复至主host:0 手动恢复至主host:1 (Optional) */
	SwitchRecoverRule int `json:"switchRecoverRule"`

	/* 1为A记录，2为CNAME (Optional) */
	Type int `json:"type"`

	/* 使用记录，host_value 0，ip_backup_01 1，ip_backup_02 2，cname_backup 3 (Optional) */
	UsedType int `json:"usedType"`

	/* 备用地址及其状态列表 (Optional) */
	BackupAddressList []BackupAddressesInfo `json:"backupAddressList"`

	/* 探测请求携带自定义头域及其域值列表 (Optional) */
	RequestHeaders []HttpHeader `json:"requestHeaders"`

	/* 探测响应Body体中包含的字符串 (Optional) */
	ResponseBodyMatch string `json:"responseBodyMatch"`

	/* 探测响应码范围列表 (Optional) */
	ResponseCodeRanges []HttpResponseCodeRange `json:"responseCodeRanges"`

	/* 正在使用的有效解析地址 (Optional) */
	EffectAddr string `json:"effectAddr"`
}
